@@ -15,15 +15,18 @@
 
 import abc
 import ast
-import base64
 import csv
-import json
+import configparser
+import io
 import re
-import six
-from six.moves.configparser import SafeConfigParser
+import sys
+
 import xmltodict
 import yaml
 
+from oslo_serialization import base64
+from oslo_serialization import jsonutils
+from oslo_utils import strutils
 
 from trove.common import utils as trove_utils
 
@@ -70,7 +73,7 @@ class StringConverter(object):
         # Return known mappings and quoted strings right away.
         if value in self._object_mappings:
             return self._object_mappings[value]
-        elif (isinstance(value, six.string_types) and
+        elif (isinstance(value, str) and
               re.match("^'(.*)'|\"(.*)\"$", value)):
             return value
 
@@ -80,8 +83,7 @@ class StringConverter(object):
             return value
 
 
-@six.add_metaclass(abc.ABCMeta)
-class StreamCodec(object):
+class StreamCodec(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def serialize(self, data):
@@ -198,14 +200,14 @@ class IniCodec(StreamCodec):
 
     def serialize(self, dict_data):
         parser = self._init_config_parser(dict_data)
-        output = six.StringIO()
+        output = io.StringIO()
         parser.write(output)
 
         return output.getvalue()
 
     def deserialize(self, stream):
         parser = self._init_config_parser()
-        parser.readfp(self._pre_parse(stream))
+        parser.read_file(self._pre_parse(stream))
 
         return {s: {k:
                     StringConverter({None: self._default_value}).to_objects(v)
@@ -213,8 +215,8 @@ class IniCodec(StreamCodec):
                 for s in parser.sections()}
 
     def _pre_parse(self, stream):
-        buf = six.StringIO()
-        for line in six.StringIO(stream):
+        buf = io.StringIO()
+        for line in io.StringIO(stream):
             # Ignore commented lines.
             if not line.startswith(self._comment_markers):
                 # Strip leading and trailing whitespaces from each line.
@@ -227,7 +229,11 @@ class IniCodec(StreamCodec):
         return buf
 
     def _init_config_parser(self, sections=None):
-        parser = SafeConfigParser(allow_no_value=True)
+        # SafeConfigParser was deprecated in Python 3.2
+        if sys.version_info >= (3, 2):
+            parser = configparser.ConfigParser(allow_no_value=True)
+        else:
+            parser = configparser.SafeConfigParser(allow_no_value=True)
         if sections:
             for section in sections:
                 parser.add_section(section)
@@ -276,7 +282,7 @@ class PropertiesCodec(StreamCodec):
         :type comment_markers:    list
 
         :param unpack_singletons: Whether to unpack singleton collections
-                                  (collections with only a single value).
+                                  (collections with only a single item).
         :type unpack_singletons:  boolean
 
         :param string_mappings:   User-defined string representations of
@@ -289,7 +295,7 @@ class PropertiesCodec(StreamCodec):
         self._unpack_singletons = unpack_singletons
 
     def serialize(self, dict_data):
-        output = six.StringIO()
+        output = io.StringIO()
         writer = csv.writer(output, delimiter=self._delimiter,
                             quoting=self.QUOTING_MODE,
                             strict=self.STRICT_MODE,
@@ -301,7 +307,7 @@ class PropertiesCodec(StreamCodec):
         return output.getvalue()
 
     def deserialize(self, stream):
-        reader = csv.reader(six.StringIO(stream),
+        reader = csv.reader(io.StringIO(stream),
                             delimiter=self._delimiter,
                             quoting=self.QUOTING_MODE,
                             strict=self.STRICT_MODE,
@@ -316,9 +322,14 @@ class PropertiesCodec(StreamCodec):
                 key = row[0].strip()
                 # Ignore comment lines.
                 if not key.strip().startswith(self._comment_markers):
-                    items = self._string_converter.to_objects(
+                    # NOTE(zhaochao): a list object is expected for
+                    # trove_utils.unpack_singleton, however in python3
+                    # map objects won't be treated as lists, so we
+                    # convert the result of StringConverter.to_objects
+                    # to a list explicitly.
+                    items = list(self._string_converter.to_objects(
                         [v if v else None for v in
-                         map(self._strip_comments, row[1:])])
+                         map(self._strip_comments, row[1:])]))
                     current = data_dict.get(key)
                     if current is not None:
                         current.append(trove_utils.unpack_singleton(items)
@@ -328,7 +339,14 @@ class PropertiesCodec(StreamCodec):
 
         if self._unpack_singletons:
             # Unpack singleton values.
-            for k, v in data_dict.items():
+            # NOTE(zhaochao): In Python 3.x, dict.items() returns a view
+            # object, which will reflect the changes of the dict itself:
+            # https://docs.python.org/3/library/stdtypes.html#dict-views
+            # This means as we're changing the dict, dict.items() cannot
+            # guarantee we're safely iterating all entries in the dict.
+            # Manually converting the result of dict.items() to a list will
+            # fix.
+            for k, v in list(data_dict.items()):
                 data_dict.update({k: trove_utils.unpack_singleton(v)})
 
         return data_dict
@@ -352,9 +370,14 @@ class PropertiesCodec(StreamCodec):
                     header, self._string_converter.to_strings(items)))
         else:
             # This is a single-row property with only one argument.
+            # Note(zhaochao): csv.writerows expects a list object before
+            # python 3.5, but map objects won't be treated as lists in
+            # python 3, so we explicitly convert the result of
+            # StringConverter.to_strings to a list here to support py34
+            # unittests.
             rows.append(
-                self._string_converter.to_strings(
-                    self._to_list(header, items)))
+                list(self._string_converter.to_strings(
+                    self._to_list(header, items))))
 
         return rows
 
@@ -426,7 +449,14 @@ class KeyValueCodec(StreamCodec):
         return self._line_terminator.join(lines)
 
     def deserialize(self, stream):
-        lines = stream.split(self._line_terminator)
+        # Note(zhaochao): In Python 3, when files are opened in text mode,
+        # newlines will be translated to '\n' by default, so we just split
+        # the stream by '\n'.
+        if sys.version_info[0] >= 3:
+            lines = stream.split('\n')
+        else:
+            lines = stream.split(self._line_terminator)
+
         result = {}
         for line in lines:
             line = line.lstrip().rstrip()
@@ -439,6 +469,8 @@ class KeyValueCodec(StreamCodec):
                 v = v.lstrip(
                     self._value_quote_char).rstrip(
                     self._value_quote_char)
+            elif v.lower() in ['true', 'false']:
+                v = strutils.bool_from_string(v.lower())
             else:
                 # remove trailing comments
                 v = re.sub('%s.*$' % self._comment_marker, '', v)
@@ -479,10 +511,13 @@ class KeyValueCodec(StreamCodec):
 class JsonCodec(StreamCodec):
 
     def serialize(self, dict_data):
-        return json.dumps(dict_data)
+        return jsonutils.dumps(dict_data)
 
     def deserialize(self, stream):
-        return json.load(six.StringIO(stream))
+        if type(stream) == str:
+            return jsonutils.load(io.StringIO(stream))
+        if type(stream) == bytes:
+            return jsonutils.load(io.BytesIO(stream))
 
 
 class Base64Codec(StreamCodec):
@@ -492,22 +527,14 @@ class Base64Codec(StreamCodec):
     binary data before writing to a file as well.
     """
 
-    def serialize(self, data):
+    # NOTE(zhaochao): migrate to oslo_serialization.base64 to serialize(return
+    # a text object) and deserialize(return a bytes object) data.
 
-        try:
-            # py27str - if we've got text data, this should encode it
-            # py27aa/py34aa - if we've got a bytearray, this should work too
-            encoded = str(base64.b64encode(data).decode('utf-8'))
-        except TypeError:
-            # py34str - convert to bytes first, then we can encode
-            data_bytes = bytes([ord(item) for item in data])
-            encoded = base64.b64encode(data_bytes).decode('utf-8')
-        return encoded
+    def serialize(self, data):
+        return base64.encode_as_text(data)
 
     def deserialize(self, stream):
-
-        # py27 & py34 seem to understand bytearray the same
-        return bytearray([item for item in base64.b64decode(stream)])
+        return base64.decode_as_bytes(stream)
 
 
 class XmlCodec(StreamCodec):
