@@ -14,25 +14,23 @@
 #    under the License.
 """I totally stole most of this from melange, thx guys!!!"""
 
-import collections
-import datetime
+from collections import abc
 import inspect
 import os
+import shlex
 import shutil
-import time
 import uuid
+import urllib.parse as urlparse
 
 from eventlet.timeout import Timeout
 import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_service import loopingcall
+from oslo_utils.encodeutils import safe_encode
 from oslo_utils import importutils
 from oslo_utils import strutils
-from oslo_utils import timeutils
-from passlib import utils as passlib_utils
-import six
-import six.moves.urllib.parse as urlparse
+from passlib import pwd
 
 from trove.common import cfg
 from trove.common import exception
@@ -46,19 +44,21 @@ import_object = importutils.import_object
 import_module = importutils.import_module
 bool_from_string = strutils.bool_from_string
 execute = processutils.execute
-isotime = timeutils.isotime
 
 
 def build_jinja_environment():
-    env = jinja2.Environment(loader=jinja2.ChoiceLoader([
-        jinja2.FileSystemLoader(CONF.template_path),
-        jinja2.PackageLoader("trove", "templates")
-    ]))
+    env = jinja2.Environment(
+        autoescape=True,
+        loader=jinja2.ChoiceLoader([
+            jinja2.FileSystemLoader(CONF.template_path),
+            jinja2.PackageLoader("trove", "templates")
+        ]))
     # Add some basic operation not built-in.
     env.globals['max'] = max
     env.globals['min'] = min
     return env
 
+
 ENV = build_jinja_environment()
 
 
@@ -97,10 +97,6 @@ def generate_uuid():
     return str(uuid.uuid4())
 
 
-def utcnow():
-    return datetime.datetime.utcnow()
-
-
 def raise_if_process_errored(process, exception):
     try:
         err = process.stderr.read()
@@ -123,7 +119,7 @@ class cached_property(object):
 
     Taken from : https://github.com/nshah/python-memoize
     The function wrapped is called the first time to retrieve the result
-    and than that calculated result is used the next time you access
+    and then that calculated result is used the next time you access
     the value:
 
         class Foo(object):
@@ -189,31 +185,43 @@ class MethodInspector(object):
 
 
 def build_polling_task(retriever, condition=lambda value: value,
-                       sleep_time=1, time_out=None):
-    start_time = time.time()
+                       sleep_time=1, time_out=0, initial_delay=0):
+    """Run a function in a loop with backoff on error.
+
+    The condition function runs based on the retriever function result.
+    """
 
     def poll_and_check():
         obj = retriever()
         if condition(obj):
             raise loopingcall.LoopingCallDone(retvalue=obj)
-        if time_out is not None and time.time() - start_time > time_out:
-            raise exception.PollTimeOut
 
-    return loopingcall.FixedIntervalLoopingCall(
-        f=poll_and_check).start(sleep_time, initial_delay=False)
+    call = loopingcall.BackOffLoopingCall(f=poll_and_check)
+    return call.start(initial_delay=initial_delay,
+                      starting_interval=sleep_time,
+                      max_interval=30, timeout=time_out)
+
+
+def wait_for_task(polling_task):
+    """Waits for the task until it is finished"""
+    try:
+        return polling_task.wait()
+    except loopingcall.LoopingCallTimeOut:
+        raise exception.PollTimeOut
 
 
 def poll_until(retriever, condition=lambda value: value,
-               sleep_time=1, time_out=None):
+               sleep_time=3, time_out=0, initial_delay=0):
     """Retrieves object until it passes condition, then returns it.
 
     If time_out_limit is passed in, PollTimeOut will be raised once that
     amount of time is eclipsed.
 
     """
-
-    return build_polling_task(retriever, condition=condition,
-                              sleep_time=sleep_time, time_out=time_out).wait()
+    task = build_polling_task(retriever, condition=condition,
+                              sleep_time=sleep_time, time_out=time_out,
+                              initial_delay=initial_delay)
+    return wait_for_task(task)
 
 
 # Copied from nova.api.openstack.common in the old code.
@@ -231,7 +239,7 @@ def get_id_from_href(href):
 
 
 def execute_with_timeout(*args, **kwargs):
-    time = kwargs.pop('timeout', 30)
+    time = kwargs.pop('timeout', CONF.command_process_timeout)
     log_output_on_error = kwargs.pop('log_output_on_error', False)
 
     timeout = Timeout(time)
@@ -240,24 +248,29 @@ def execute_with_timeout(*args, **kwargs):
     except exception.ProcessExecutionError as e:
         if log_output_on_error:
             LOG.error(
-                _("Command '%(cmd)s' failed. %(description)s "
-                  "Exit code: %(exit_code)s\nstderr: %(stderr)s\n"
-                  "stdout: %(stdout)s") %
+                ("Command '%(cmd)s' failed. %(description)s "
+                 "Exit code: %(exit_code)s\nstderr: %(stderr)s\n"
+                 "stdout: %(stdout)s"),
                 {'cmd': e.cmd, 'description': e.description or '',
                  'exit_code': e.exit_code, 'stderr': e.stderr,
                  'stdout': e.stdout})
         raise
     except Timeout as t:
         if t is not timeout:
-            LOG.error(_("Got a timeout but not the one expected."))
+            LOG.error("Got a timeout but not the one expected.")
             raise
         else:
-            msg = (_("Time out after waiting "
-                     "%(time)s seconds when running proc: %(args)s"
-                     " %(kwargs)s.") % {'time': time, 'args': args,
-                                        'kwargs': kwargs})
-            LOG.error(msg)
-            raise exception.ProcessExecutionError(msg)
+            log_fmt = ("Time out after waiting "
+                       "%(time)s seconds when running proc: %(args)s"
+                       " %(kwargs)s.")
+            exc_fmt = _("Time out after waiting "
+                        "%(time)s seconds when running proc: %(args)s"
+                        " %(kwargs)s.")
+            msg_content = {
+                'time': time, 'args': args,
+                'kwargs': kwargs}
+            LOG.error(log_fmt, msg_content)
+            raise exception.ProcessExecutionError(exc_fmt % msg_content)
     finally:
         timeout.cancel()
 
@@ -285,7 +298,7 @@ def generate_random_password(password_length=None):
         password_length or
         cfg.get_configuration_property('default_password_length')
     )
-    return passlib_utils.generate_password(size=password_length)
+    return pwd.genword(length=password_length)
 
 
 def try_recover(func):
@@ -297,22 +310,12 @@ def try_recover(func):
             if recover_func is not None:
                 recover_func(func)
             else:
-                LOG.debug("No recovery method defined for %(func)s" % {
+                LOG.debug("No recovery method defined for %(func)s", {
                           'func': func.__name__})
             raise
     return _decorator
 
 
-def gen_ports(portstr):
-    from_port, sep, to_port = portstr.partition('-')
-    if not (to_port and from_port):
-        if not sep:
-            to_port = from_port
-    if int(from_port) > int(to_port):
-        raise ValueError
-    return from_port, to_port
-
-
 def unpack_singleton(container):
     """Unpack singleton collections.
 
@@ -329,8 +332,8 @@ def unpack_singleton(container):
 def is_collection(item):
     """Return True is a given item is an iterable collection, but not a string.
     """
-    return (isinstance(item, collections.Iterable) and
-            not isinstance(item, (bytes, six.text_type)))
+    return (isinstance(item, abc.Iterable) and
+            not isinstance(item, (bytes, str)))
 
 
 def format_output(message, format_len=79, truncate_len=None, replace_index=0):
@@ -396,3 +399,38 @@ def to_mb(bytes):
     size = bytes / 1024.0 ** 2
     # Make sure we don't return 0.0 if the size is greater than 0
     return max(round(size, 2), 0.01)
+
+
+def req_to_text(req):
+    """
+    We do a lot request logging for debug, but if the value of one
+    request header is encoded in utf-8, an UnicodeEncodeError will
+    be raised. So we should carefully encode request headers.
+
+    To be consitent with webob, main procedures are copied from
+    webob.Request.as_bytes.
+    """
+    url = req.url
+    host = req.host_url
+    assert url.startswith(host)
+    url = url[len(host):]
+    parts = [safe_encode('%s %s %s' % (req.method, url, req.http_version))]
+
+    for k, v in sorted(req.headers.items()):
+        header = safe_encode('%s: %s' % (k, v))
+        parts.append(header)
+
+    if req.body:
+        parts.extend([b'', safe_encode(req.body)])
+
+    return b'\r\n'.join(parts).decode(req.charset)
+
+
+def validate_command(string):
+    """
+    Check if the string is legal for command
+
+    raise invalidvalue if illegal
+    """
+    if string != shlex.quote(string):
+        raise exception.InvalidValue(value=string)
