@@ -13,40 +13,46 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ipaddress
+
 from oslo_log import log as logging
 from oslo_utils import strutils
-import webob.exc
 
 from trove.backup.models import Backup as backup_model
 from trove.backup import views as backup_views
 import trove.common.apischema as apischema
 from trove.common import cfg
+from trove.common import clients
 from trove.common import exception
+from trove.common import glance as common_glance
 from trove.common.i18n import _
-from trove.common.i18n import _LI
+from trove.common import neutron
 from trove.common import notification
 from trove.common.notification import StartNotification
 from trove.common import pagination
-from trove.common.remote import create_guest_client
+from trove.common import policy
 from trove.common import utils
 from trove.common import wsgi
-from trove.datastore import models as datastore_models
+from trove.datastore import models as ds_models
 from trove.extensions.mysql.common import populate_users
 from trove.extensions.mysql.common import populate_validated_databases
 from trove.instance import models, views
 from trove.module import models as module_models
 from trove.module import views as module_views
 
-
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
 class InstanceController(wsgi.Controller):
-
     """Controller for instance functionality."""
     schemas = apischema.instance.copy()
 
+    @classmethod
+    def authorize_instance_action(cls, context, instance_rule_name, instance):
+        policy.authorize_on_target(context, 'instance:%s' % instance_rule_name,
+                                   {'tenant': instance.tenant_id})
+
     @classmethod
     def get_action_schema(cls, body, action_schema):
         action_type = list(body.keys())[0]
@@ -68,7 +74,7 @@ class InstanceController(wsgi.Controller):
     def action(self, req, body, tenant_id, id):
         """
         Handles requests that modify existing instances in some manner. Actions
-        could include 'resize', 'restart', 'reset_password'
+        could include 'resize', 'restart'
         :param req: http request object
         :param body: deserialized body of the request as a dict
         :param tenant_id: the tenant id for whom owns the instance
@@ -81,9 +87,8 @@ class InstanceController(wsgi.Controller):
         _actions = {
             'restart': self._action_restart,
             'resize': self._action_resize,
-            'reset_password': self._action_reset_password,
             'promote_to_replica_source':
-            self._action_promote_to_replica_source,
+                self._action_promote_to_replica_source,
             'eject_replica_source': self._action_eject_replica_source,
             'reset_status': self._action_reset_status,
         }
@@ -93,10 +98,11 @@ class InstanceController(wsgi.Controller):
             if key in _actions:
                 selected_action = _actions[key]
                 action_name = key
-        LOG.info(_LI("Performing %(action_name)s action against "
-                     "instance %(instance_id)s for tenant '%(tenant_id)s'"),
+        LOG.info("Performing %(action_name)s action against "
+                 "instance %(instance_id)s for tenant %(tenant_id)s, "
+                 "body: %(body)s",
                  {'action_name': action_name, 'instance_id': id,
-                  'tenant_id': tenant_id})
+                  'tenant_id': tenant_id, 'body': body})
         needs_server = True
         if action_name in ['reset_status']:
             needs_server = False
@@ -106,6 +112,7 @@ class InstanceController(wsgi.Controller):
     def _action_restart(self, context, req, instance, body):
         context.notification = notification.DBaaSInstanceRestart(context,
                                                                  request=req)
+        self.authorize_instance_action(context, 'restart', instance)
         with StartNotification(context, instance_id=instance.id):
             instance.restart()
         return wsgi.Result(None, 202)
@@ -136,6 +143,8 @@ class InstanceController(wsgi.Controller):
     def _action_resize_volume(self, context, req, instance, volume):
         context.notification = notification.DBaaSInstanceResizeVolume(
             context, request=req)
+        self.authorize_instance_action(context, 'resize_volume', instance)
+
         with StartNotification(context, instance_id=instance.id,
                                new_size=volume['size']):
             instance.resize_volume(volume['size'])
@@ -144,16 +153,17 @@ class InstanceController(wsgi.Controller):
     def _action_resize_flavor(self, context, req, instance, flavorRef):
         context.notification = notification.DBaaSInstanceResizeInstance(
             context, request=req)
+        self.authorize_instance_action(context, 'resize_flavor', instance)
+
         new_flavor_id = utils.get_id_from_href(flavorRef)
         with StartNotification(context, instance_id=instance.id,
                                new_flavor_id=new_flavor_id):
             instance.resize_flavor(new_flavor_id)
         return wsgi.Result(None, 202)
 
-    def _action_reset_password(self, context, instance, body):
-        raise webob.exc.HTTPNotImplemented()
-
     def _action_promote_to_replica_source(self, context, req, instance, body):
+        self.authorize_instance_action(
+            context, 'promote_to_replica_source', instance)
         context.notification = notification.DBaaSInstanceEject(context,
                                                                request=req)
         with StartNotification(context, instance_id=instance.id):
@@ -161,6 +171,8 @@ class InstanceController(wsgi.Controller):
         return wsgi.Result(None, 202)
 
     def _action_eject_replica_source(self, context, req, instance, body):
+        self.authorize_instance_action(
+            context, 'eject_replica_source', instance)
         context.notification = notification.DBaaSInstancePromote(context,
                                                                  request=req)
         with StartNotification(context, instance_id=instance.id):
@@ -168,35 +180,63 @@ class InstanceController(wsgi.Controller):
         return wsgi.Result(None, 202)
 
     def _action_reset_status(self, context, req, instance, body):
+        if 'force_delete' in body['reset_status']:
+            self.authorize_instance_action(context, 'force_delete', instance)
+        else:
+            self.authorize_instance_action(
+                context, 'reset_status', instance)
         context.notification = notification.DBaaSInstanceResetStatus(
             context, request=req)
         with StartNotification(context, instance_id=instance.id):
             instance.reset_status()
 
-            LOG.debug("Failing backups for instance %s." % instance.id)
+            LOG.debug("Failing backups for instance %s.", instance.id)
             backup_model.fail_for_instance(instance.id)
 
         return wsgi.Result(None, 202)
 
     def index(self, req, tenant_id):
         """Return all instances."""
-        LOG.info(_LI("Listing database instances for tenant '%s'"), tenant_id)
+        LOG.info("Listing database instances for tenant '%s'", tenant_id)
         LOG.debug("req : '%s'\n\n", req)
+        context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'instance:index')
+        instances = self._get_instances(req, instance_view=views.InstanceView)
+        return wsgi.Result(instances, 200)
+
+    def detail(self, req, tenant_id):
+        """Return all instances with details."""
+        LOG.info("Listing database instances with details for tenant '%s'",
+                 tenant_id)
+        LOG.debug("req : '%s'\n\n", req)
+        context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'instance:detail')
+        instances = self._get_instances(req,
+                                        instance_view=views.InstanceDetailView)
+        return wsgi.Result(instances, 200)
+
+    def _get_instances(self, req, instance_view):
         context = req.environ[wsgi.CONTEXT_KEY]
         clustered_q = req.GET.get('include_clustered', '').lower()
         include_clustered = clustered_q == 'true'
-        servers, marker = models.Instances.load(context, include_clustered)
-        view = views.InstancesView(servers, req=req)
+        instances, marker = models.Instances.load(context, include_clustered)
+        view = views.InstancesView(instances,
+                                   item_view=instance_view,
+                                   req=req)
         paged = pagination.SimplePaginatedDataView(req.url, 'instances', view,
                                                    marker)
-        return wsgi.Result(paged.data(), 200)
+        return paged.data()
 
     def backups(self, req, tenant_id, id):
         """Return all backups for the specified instance."""
-        LOG.info(_LI("Listing backups for instance '%s'"),
+        LOG.info("Listing backups for instance '%s'",
                  id)
         LOG.debug("req : '%s'\n\n", req)
         context = req.environ[wsgi.CONTEXT_KEY]
+
+        instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'backups', instance)
+
         backups, marker = backup_model.list_for_instance(context, id)
         view = backup_views.BackupViews(backups)
         paged = pagination.SimplePaginatedDataView(req.url, 'backups', view,
@@ -205,25 +245,28 @@ class InstanceController(wsgi.Controller):
 
     def show(self, req, tenant_id, id):
         """Return a single instance."""
-        LOG.info(_LI("Showing database instance '%(instance_id)s' for tenant "
-                     "'%(tenant_id)s'"),
+        LOG.info("Showing database instance '%(instance_id)s' for tenant "
+                 "'%(tenant_id)s'",
                  {'instance_id': id, 'tenant_id': tenant_id})
         LOG.debug("req : '%s'\n\n", req)
 
         context = req.environ[wsgi.CONTEXT_KEY]
-        server = models.load_instance_with_info(models.DetailInstance,
-                                                context, id)
-        return wsgi.Result(views.InstanceDetailView(server,
-                                                    req=req).data(), 200)
+        instance = models.load_instance_with_info(models.DetailInstance,
+                                                  context, id)
+        self.authorize_instance_action(context, 'show', instance)
+        return wsgi.Result(
+            views.InstanceDetailView(instance, req=req).data(), 200
+        )
 
     def delete(self, req, tenant_id, id):
         """Delete a single instance."""
-        LOG.info(_LI("Deleting database instance '%(instance_id)s' for tenant "
-                     "'%(tenant_id)s'"),
+        LOG.info("Deleting database instance '%(instance_id)s' for tenant "
+                 "'%(tenant_id)s'",
                  {'instance_id': id, 'tenant_id': tenant_id})
         LOG.debug("req : '%s'\n\n", req)
         context = req.environ[wsgi.CONTEXT_KEY]
         instance = models.load_any_instance(context, id)
+        self.authorize_instance_action(context, 'delete', instance)
         context.notification = notification.DBaaSInstanceDelete(
             context, request=req)
         with StartNotification(context, instance_id=instance.id):
@@ -240,24 +283,146 @@ class InstanceController(wsgi.Controller):
             instance.delete()
         return wsgi.Result(None, 202)
 
+    def _check_nic(self, context, nic):
+        """Check user provided nic.
+
+        :param context: User context.
+        :param nic: A dict may contain network_id(net-id), subnet_id or
+            ip_address.
+        """
+        neutron_client = clients.create_neutron_client(context)
+        network_id = nic.get('network_id', nic.get('net-id'))
+        subnet_id = nic.get('subnet_id')
+        ip_address = nic.get('ip_address')
+
+        if not network_id and not subnet_id:
+            raise exception.NetworkNotProvided(resource='network or subnet')
+
+        if not subnet_id and ip_address:
+            raise exception.NetworkNotProvided(resource='subnet')
+
+        if subnet_id:
+            actual_network = neutron_client.show_subnet(
+                subnet_id)['subnet']['network_id']
+            if network_id and actual_network != network_id:
+                raise exception.SubnetNotFound(subnet_id=subnet_id,
+                                               network_id=network_id)
+            network_id = actual_network
+
+        nic['network_id'] = network_id
+        nic.pop('net-id', None)
+
+        self._check_network_overlap(context, network_id, subnet_id)
+
+    def _check_network_overlap(self, context, user_network=None,
+                               user_subnet=None):
+        """Check if the network contains IP address belongs to reserved
+        network.
+
+        :param context: User context.
+        :param user_network: Network ID.
+        :param user_subnet: Subnet ID.
+        """
+        neutron_client = clients.create_neutron_client(context)
+        user_cidrs = neutron.get_subnet_cidrs(neutron_client, user_network,
+                                              user_subnet)
+
+        reserved_cidrs = CONF.reserved_network_cidrs
+        mgmt_cidrs = neutron.get_mamangement_subnet_cidrs(neutron_client)
+        reserved_cidrs.extend(mgmt_cidrs)
+
+        LOG.debug("Cidrs of the user network: %s, cidrs of the reserved "
+                  "network: %s", user_cidrs, reserved_cidrs)
+
+        for user_cidr in user_cidrs:
+            user_net = ipaddress.ip_network(user_cidr)
+            for reserved_cidr in reserved_cidrs:
+                res_net = ipaddress.ip_network(reserved_cidr)
+                if user_net.overlaps(res_net):
+                    raise exception.NetworkConflict()
+
     def create(self, req, body, tenant_id):
-        # TODO(hub-cap): turn this into middleware
-        LOG.info(_LI("Creating a database instance for tenant '%s'"),
+        LOG.info("Creating a database instance for tenant '%s'",
                  tenant_id)
         LOG.debug("req : '%s'\n\n", strutils.mask_password(req))
         LOG.debug("body : '%s'\n\n", strutils.mask_password(body))
         context = req.environ[wsgi.CONTEXT_KEY]
-        context.notification = notification.DBaaSInstanceCreate(context,
-                                                                request=req)
-        datastore_args = body['instance'].get('datastore', {})
-        datastore, datastore_version = (
-            datastore_models.get_datastore_version(**datastore_args))
-        image_id = datastore_version.image_id
+        policy.authorize_on_tenant(context, 'instance:create')
+        context.notification = notification.DBaaSInstanceCreate(
+            context, request=req)
+
         name = body['instance']['name']
-        flavor_ref = body['instance']['flavorRef']
+        slave_of_id = body['instance'].get('replica_of')
+        replica_count = body['instance'].get('replica_count')
+        flavor_ref = body['instance'].get('flavorRef')
+        datastore_args = body['instance'].get('datastore', {})
+        volume_info = body['instance'].get('volume', {})
+        availability_zone = body['instance'].get('availability_zone')
+        nics = body['instance'].get('nics', [])
+        locality = body['instance'].get('locality')
+        region_name = body['instance'].get(
+            'region_name', CONF.service_credentials.region_name
+        )
+        access = body['instance'].get('access', None)
+
+        if slave_of_id:
+            if flavor_ref:
+                msg = 'Cannot specify flavor when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            if datastore_args:
+                msg = 'Cannot specify datastore when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            if volume_info:
+                msg = 'Cannot specify volume when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            if locality:
+                msg = 'Cannot specify locality when creating replicas.'
+                raise exception.BadRequest(message=msg)
+            backup_model.verify_swift_auth_token(context)
+        else:
+            if replica_count and replica_count > 1:
+                msg = (f"Replica count only valid when creating replicas. "
+                       f"Cannot create {replica_count} instances.")
+                raise exception.BadRequest(message=msg)
+
         flavor_id = utils.get_id_from_href(flavor_ref)
 
-        configuration = self._configuration_parse(context, body)
+        if volume_info:
+            volume_size = int(volume_info.get('size'))
+            volume_type = volume_info.get('type')
+        else:
+            volume_size = None
+            volume_type = None
+
+        if slave_of_id:
+            try:
+                replica_source = models.DBInstance.find_by(
+                    context, id=slave_of_id, deleted=False)
+                flavor_id = replica_source.flavor_id
+            except exception.ModelNotFoundError:
+                LOG.error(f"Cannot create a replica of {slave_of_id} as that "
+                          f"instance could not be found.")
+                raise exception.NotFound(uuid=slave_of_id)
+            if replica_source.slave_of_id:
+                raise exception.Forbidden(
+                    f"Cannot create a replica of a replica {slave_of_id}")
+
+            datastore_version = ds_models.DatastoreVersion.load_by_uuid(
+                replica_source.datastore_version_id)
+            datastore = ds_models.Datastore.load(
+                datastore_version.datastore_id)
+        else:
+            datastore, datastore_version = ds_models.get_datastore_version(
+                **datastore_args)
+
+        # If only image_tags is configured in the datastore version, get
+        # the image ID using the tags.
+        glance_client = clients.create_glance_client(context)
+        image_id = common_glance.get_image_id(
+            glance_client, datastore_version.image_id,
+            datastore_version.image_tags)
+        LOG.info(f'Using image {image_id} for creating instance')
+
         databases = populate_validated_databases(
             body['instance'].get('databases', []))
         database_names = [database.get('_name', '') for database in databases]
@@ -266,15 +431,29 @@ class InstanceController(wsgi.Controller):
             users = populate_users(body['instance'].get('users', []),
                                    database_names)
         except ValueError as ve:
-            raise exception.BadRequest(msg=ve)
+            raise exception.BadRequest(message=str(ve))
+        if slave_of_id and (databases or users):
+            raise exception.ReplicaCreateWithUsersDatabasesError()
 
-        if 'volume' in body['instance']:
-            volume_info = body['instance']['volume']
-            volume_size = int(volume_info['size'])
-            volume_type = volume_info.get('type')
-        else:
-            volume_size = None
-            volume_type = None
+        configuration = self._configuration_parse(context, body)
+        modules = body['instance'].get('modules')
+
+        # The following operations have their own API calls.
+        # We need to make sure the same policies are enforced when
+        # creating an instance.
+        # i.e. if attaching configuration group to an existing instance is not
+        # allowed, it should not be possible to create a new instance with the
+        # group attached either
+        if configuration:
+            policy.authorize_on_tenant(context, 'instance:update')
+        if modules:
+            policy.authorize_on_tenant(context, 'instance:module_apply')
+        if users:
+            policy.authorize_on_tenant(
+                context, 'instance:extension:user:create')
+        if databases:
+            policy.authorize_on_tenant(
+                context, 'instance:extension:database:create')
 
         if 'restorePoint' in body['instance']:
             backupRef = body['instance']['restorePoint']['backupRef']
@@ -282,15 +461,16 @@ class InstanceController(wsgi.Controller):
         else:
             backup_id = None
 
-        availability_zone = body['instance'].get('availability_zone')
-        nics = body['instance'].get('nics')
+        # Only 1 nic is allowed as defined in API jsonschema.
+        # Use list just for backward compatibility.
+        if len(nics) > 0:
+            nic = nics[0]
+            LOG.info('Checking user provided instance network %s', nic)
+            if slave_of_id and nic.get('ip_address'):
+                msg = "Cannot specify IP address when creating replicas."
+                raise exception.BadRequest(message=msg)
+            self._check_nic(context, nic)
 
-        slave_of_id = body['instance'].get('replica_of',
-                                           # also check for older name
-                                           body['instance'].get('slave_of'))
-        replica_count = body['instance'].get('replica_count')
-        modules = body['instance'].get('modules')
-        locality = body['instance'].get('locality')
         if locality:
             locality_domain = ['affinity', 'anti-affinity']
             locality_domain_msg = ("Invalid locality '%s'. "
@@ -298,12 +478,7 @@ class InstanceController(wsgi.Controller):
                                    (locality,
                                     "', '".join(locality_domain)))
             if locality not in locality_domain:
-                raise exception.BadRequest(msg=locality_domain_msg)
-            if slave_of_id:
-                dupe_locality_msg = (
-                    'Cannot specify locality when adding replicas to existing '
-                    'master.')
-                raise exception.BadRequest(msg=dupe_locality_msg)
+                raise exception.BadRequest(message=locality_domain_msg)
 
         instance = models.Instance.create(context, name, flavor_id,
                                           image_id, databases, users,
@@ -314,7 +489,9 @@ class InstanceController(wsgi.Controller):
                                           replica_count=replica_count,
                                           volume_type=volume_type,
                                           modules=modules,
-                                          locality=locality)
+                                          locality=locality,
+                                          region_name=region_name,
+                                          access=access)
 
         view = views.InstanceDetailView(instance, req=req)
         return wsgi.Result(view.data(), 200)
@@ -325,15 +502,15 @@ class InstanceController(wsgi.Controller):
             if configuration_ref:
                 configuration_id = utils.get_id_from_href(configuration_ref)
                 return configuration_id
+            return ""
 
     def _modify_instance(self, context, req, instance, **kwargs):
         if 'detach_replica' in kwargs and kwargs['detach_replica']:
-            LOG.debug("Detaching replica from source.")
             context.notification = notification.DBaaSInstanceDetach(
                 context, request=req)
             with StartNotification(context, instance_id=instance.id):
                 instance.detach_replica()
-        if 'configuration_id' in kwargs:
+        elif 'configuration_id' in kwargs:
             if kwargs['configuration_id']:
                 context.notification = (
                     notification.DBaaSInstanceAttachConfiguration(context,
@@ -341,96 +518,126 @@ class InstanceController(wsgi.Controller):
                 configuration_id = kwargs['configuration_id']
                 with StartNotification(context, instance_id=instance.id,
                                        configuration_id=configuration_id):
-                    instance.assign_configuration(configuration_id)
+                    instance.attach_configuration(configuration_id)
             else:
                 context.notification = (
                     notification.DBaaSInstanceDetachConfiguration(context,
                                                                   request=req))
                 with StartNotification(context, instance_id=instance.id):
-                    instance.unassign_configuration()
-        if 'datastore_version' in kwargs:
-            datastore_version = datastore_models.DatastoreVersion.load(
+                    instance.detach_configuration()
+        elif 'datastore_version' in kwargs:
+            datastore_version = ds_models.DatastoreVersion.load(
                 instance.datastore, kwargs['datastore_version'])
+
+            if datastore_version.name == instance.ds_version.name:
+                LOG.warning(f"Same datastore version {datastore_version.name} "
+                            f"for upgrading")
+                return
+
             context.notification = (
                 notification.DBaaSInstanceUpgrade(context, request=req))
             with StartNotification(context, instance_id=instance.id,
                                    datastore_version_id=datastore_version.id):
                 instance.upgrade(datastore_version)
-        if kwargs:
-            instance.update_db(**kwargs)
+        elif 'access' in kwargs:
+            instance.update_access(kwargs['access'])
 
     def update(self, req, id, body, tenant_id):
-        """Updates the instance to attach/detach configuration."""
-        LOG.info(_LI("Updating database instance '%(instance_id)s' for tenant "
-                     "'%(tenant_id)s'"),
+        """Updates the instance.
+
+        - attach/detach configuration.
+        - detach from primary.
+        - access information.
+        """
+        LOG.info("Updating database instance '%(instance_id)s' for tenant "
+                 "'%(tenant_id)s'",
                  {'instance_id': id, 'tenant_id': tenant_id})
         LOG.debug("req: %s", req)
         LOG.debug("body: %s", body)
         context = req.environ[wsgi.CONTEXT_KEY]
 
+        name = body['instance'].get('name')
+        if ((name and len(body['instance'].keys()) > 2) or
+            (not name and len(body['instance'].keys()) >= 2)):
+            raise exception.BadRequest("Only one attribute (except 'name') is "
+                                       "allowed to update.")
+
         instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'update', instance)
 
-        # Make sure args contains a 'configuration_id' argument,
         args = {}
-        args['configuration_id'] = self._configuration_parse(context, body)
+        if name:
+            instance.update_db(name=name)
+
+        detach_replica = ('replica_of' in body['instance'] or
+                          'slave_of' in body['instance'])
+        if detach_replica:
+            args['detach_replica'] = detach_replica
+
+        configuration_id = self._configuration_parse(context, body)
+        if configuration_id is not None:
+            args['configuration_id'] = configuration_id
+
+        if 'access' in body['instance']:
+            args['access'] = body['instance']['access']
+
+        if 'datastore_version' in body['instance']:
+            args['datastore_version'] = body['instance']['datastore_version']
+
         self._modify_instance(context, req, instance, **args)
         return wsgi.Result(None, 202)
 
     def edit(self, req, id, body, tenant_id):
+        """Updates the instance to set or unset one or more attributes.
+
+        Deprecated. Use update method instead.
         """
-        Updates the instance to set or unset one or more attributes.
-        """
-        LOG.info(_LI("Editing instance for tenant id %s."), tenant_id)
-        LOG.debug("req: %s", strutils.mask_password(req))
-        LOG.debug("body: %s", strutils.mask_password(body))
-        context = req.environ[wsgi.CONTEXT_KEY]
-
-        instance = models.Instance.load(context, id)
-
-        args = {}
-        args['detach_replica'] = ('replica_of' in body['instance'] or
-                                  'slave_of' in body['instance'])
-
-        if 'name' in body['instance']:
-            args['name'] = body['instance']['name']
-        if 'configuration' in body['instance']:
-            args['configuration_id'] = self._configuration_parse(context, body)
-        if 'datastore_version' in body['instance']:
-            args['datastore_version'] = body['instance'].get(
-                'datastore_version')
-
-        self._modify_instance(context, req, instance, **args)
-        return wsgi.Result(None, 202)
+        return self.update(req, id, body, tenant_id)
 
     def configuration(self, req, tenant_id, id):
         """
         Returns the default configuration template applied to the instance.
         """
-        LOG.info(_LI("Getting default configuration for instance %s"), id)
+        LOG.info("Getting default configuration for instance %s", id)
         context = req.environ[wsgi.CONTEXT_KEY]
         instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'configuration', instance)
+
         LOG.debug("Server: %s", instance)
         config = instance.get_default_configuration_template()
         LOG.debug("Default config for instance %(instance_id)s is %(config)s",
                   {'instance_id': id, 'config': config})
         return wsgi.Result(views.DefaultConfigurationView(
-                           config).data(), 200)
+            config).data(), 200)
 
     def guest_log_list(self, req, tenant_id, id):
         """Return all information about all logs for an instance."""
-        LOG.debug("Listing logs for tenant %s" % tenant_id)
+        LOG.debug("Listing logs for tenant %s", tenant_id)
         context = req.environ[wsgi.CONTEXT_KEY]
+
+        try:
+            backup_model.verify_swift_auth_token(context)
+        except exception.SwiftNotFound:
+            raise exception.LogsNotAvailable()
+
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
-        client = create_guest_client(context, id)
+        self.authorize_instance_action(context, 'guest_log_list', instance)
+        client = clients.create_guest_client(context, id)
         guest_log_list = client.guest_log_list()
         return wsgi.Result({'logs': guest_log_list}, 200)
 
     def guest_log_action(self, req, body, tenant_id, id):
         """Processes a guest log."""
-        LOG.info(_("Processing log for tenant %s"), tenant_id)
+        LOG.info("Processing log for tenant %s", tenant_id)
         context = req.environ[wsgi.CONTEXT_KEY]
+
+        try:
+            backup_model.verify_swift_auth_token(context)
+        except exception.SwiftNotFound:
+            raise exception.LogsNotAvailable()
+
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
@@ -441,7 +648,7 @@ class InstanceController(wsgi.Controller):
         discard = body.get('discard', None)
         if enable and disable:
             raise exception.BadRequest(_("Cannot enable and disable log."))
-        client = create_guest_client(context, id)
+        client = clients.create_guest_client(context, id)
         guest_log = client.guest_log_action(log_name, enable, disable,
                                             publish, discard)
         return wsgi.Result({'log': guest_log}, 200)
@@ -452,6 +659,7 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'module_list', instance)
         from_guest = bool(req.GET.get('from_guest', '').lower())
         include_contents = bool(req.GET.get('include_contents', '').lower())
         if from_guest:
@@ -463,13 +671,13 @@ class InstanceController(wsgi.Controller):
 
     def _module_list_guest(self, context, id, include_contents):
         """Return information about modules on an instance."""
-        client = create_guest_client(context, id)
+        client = clients.create_guest_client(context, id)
         result_list = client.module_list(include_contents)
         return wsgi.Result({'modules': result_list}, 200)
 
     def _module_list(self, context, id, include_contents):
-        """Return information about instnace modules."""
-        client = create_guest_client(context, id)
+        """Return information about instance modules."""
+        client = clients.create_guest_client(context, id)
         result_list = client.module_list(include_contents)
         return wsgi.Result({'modules': result_list}, 200)
 
@@ -479,16 +687,13 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'module_apply', instance)
         module_ids = [mod['id'] for mod in body.get('modules', [])]
         modules = module_models.Modules.load_by_ids(context, module_ids)
-        module_list = []
-        for module in modules:
-            module.contents = module_models.Module.deprocess_contents(
-                module.contents)
-            module_info = module_views.DetailedModuleView(module).data(
-                include_contents=True)
-            module_list.append(module_info)
-        client = create_guest_client(context, id)
+        module_models.Modules.validate(
+            modules, instance.datastore.id, instance.datastore_version.id)
+        module_list = module_views.convert_modules_to_list(modules)
+        client = clients.create_guest_client(context, id)
         result_list = client.module_apply(module_list)
         models.Instance.add_instance_modules(context, id, modules)
         return wsgi.Result({'modules': result_list}, 200)
@@ -499,12 +704,17 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'module_remove', instance)
         module = module_models.Module.load(context, module_id)
         module_info = module_views.DetailedModuleView(module).data()
-        client = create_guest_client(context, id)
+        client = clients.create_guest_client(context, id)
         client.module_remove(module_info)
-        instance_module = module_models.InstanceModule.load(
+        instance_modules = module_models.InstanceModules.load_all(
             context, instance_id=id, module_id=module_id)
-        if instance_module:
+        for instance_module in instance_modules:
             module_models.InstanceModule.delete(context, instance_module)
+            LOG.debug("Deleted IM record %(instance_module_id)s "
+                      "(instance %(id)s, module %(module_id)s).",
+                      {'instance_module_id': instance_module.id, 'id': id,
+                       'module_id': module_id})
         return wsgi.Result(None, 200)
