@@ -16,6 +16,7 @@
 
 from datetime import datetime
 import json
+import netaddr
 from time import sleep
 import uuid
 
@@ -24,21 +25,20 @@ from proboscis.asserts import assert_equal
 from proboscis.asserts import assert_not_equal
 from proboscis.asserts import assert_raises
 from proboscis.asserts import assert_true
+from proboscis.asserts import fail
 from proboscis import before_class
 from proboscis.decorators import time_out
 from proboscis import SkipTest
 from proboscis import test
-import six
 from troveclient.compat import exceptions
 
 from trove.common.utils import poll_until
-from trove.tests.api.backups import RestoreUsingBackup
+from trove import tests
 from trove.tests.api.instances import assert_unprocessable
 from trove.tests.api.instances import instance_info
 from trove.tests.api.instances import InstanceTestInfo
 from trove.tests.api.instances import TIMEOUT_INSTANCE_CREATE
 from trove.tests.api.instances import TIMEOUT_INSTANCE_DELETE
-from trove.tests.api.instances import WaitForGuestInstallationToFinish
 from trove.tests.config import CONFIG
 from trove.tests.util.check import AttrCheck
 from trove.tests.util.check import CollectionCheck
@@ -47,9 +47,6 @@ from trove.tests.util import create_dbaas_client
 from trove.tests.util.mysql import create_mysql_connection
 from trove.tests.util.users import Requirements
 
-
-GROUP = "dbaas.api.configurations"
-GROUP_CONFIG_DEFINE = "dbaas.api.configurations.define"
 CONFIG_NAME = "test_configuration"
 CONFIG_DESC = "configuration description"
 
@@ -75,16 +72,21 @@ def _is_valid_timestamp(time_string):
 
 # helper methods to validate configuration is applied to instance
 def _execute_query(host, user_name, password, query):
-    print(host, user_name, password, query)
+    print("Starting to query database, host: %s, user: %s, password: %s, "
+          "query: %s" % (host, user_name, password, query))
+
     with create_mysql_connection(host, user_name, password) as db:
         result = db.execute(query)
         return result
-    assert_true(False, "something went wrong in the sql connection")
 
 
 def _get_address(instance_id):
     result = instance_info.dbaas_admin.mgmt.instances.show(instance_id)
-    return result.ip[0]
+    try:
+        return next(str(ip) for ip in result.ip
+                    if netaddr.valid_ipv4(ip))
+    except StopIteration:
+        fail("No IPV4 ip found")
 
 
 def _test_configuration_is_applied_to_instance(instance, configuration_id):
@@ -132,7 +134,7 @@ def _test_configuration_is_applied_to_instance(instance, configuration_id):
         resp, body = instance_info.dbaas.client.last_response
         print(resp)
         print(body)
-        return json.loads(body)['type']
+        return json.loads(body.decode())['type']
 
     # check the config values are correct
     for key, value in actual_values:
@@ -172,9 +174,8 @@ class ConfigurationsTestBase(object):
         return datastore_test_configs.get("configurations", {})
 
 
-@test(depends_on_classes=[WaitForGuestInstallationToFinish],
-      runs_after=[RestoreUsingBackup],
-      groups=[GROUP, GROUP_CONFIG_DEFINE])
+@test(depends_on_groups=[tests.DBAAS_API_BACKUPS],
+      groups=[tests.DBAAS_API_CONFIGURATIONS])
 class CreateConfigurations(ConfigurationsTestBase):
 
     @test
@@ -186,7 +187,7 @@ class CreateConfigurations(ConfigurationsTestBase):
             instance_info.dbaas_datastore_version)
         resp, body = instance_info.dbaas.client.last_response
         attrcheck = AttrCheck()
-        config_parameters_dict = json.loads(body)
+        config_parameters_dict = json.loads(body.decode())
         attrcheck.contains_allowed_attrs(
             config_parameters_dict, allowed_attrs,
             msg="Configurations parameters")
@@ -220,7 +221,7 @@ class CreateConfigurations(ConfigurationsTestBase):
         print("resp: %s" % resp)
         print("body: %s" % body)
         attrcheck = AttrCheck()
-        config_parameter_dict = json.loads(body)
+        config_parameter_dict = json.loads(body.decode())
         print("config_parameter_dict: %s" % config_parameter_dict)
         attrcheck.contains_allowed_attrs(
             config_parameter_dict,
@@ -228,12 +229,12 @@ class CreateConfigurations(ConfigurationsTestBase):
             msg="Get Configuration parameter")
         assert_equal(param_name, config_parameter_dict['name'])
         with TypeCheck('ConfigurationParameter', param) as parameter:
-            parameter.has_field('name', six.string_types)
+            parameter.has_field('name', str)
             parameter.has_field('restart_required', bool)
-            parameter.has_field('max', six.integer_types)
-            parameter.has_field('min', six.integer_types)
-            parameter.has_field('type', six.string_types)
-            parameter.has_field('datastore_version_id', six.text_type)
+            parameter.has_field('max', int)
+            parameter.has_field('min', int)
+            parameter.has_field('type', str)
+            parameter.has_field('datastore_version_id', str)
 
     @test
     def test_configurations_create_invalid_values(self):
@@ -268,7 +269,7 @@ class CreateConfigurations(ConfigurationsTestBase):
 
     @test
     def test_valid_configurations_create(self):
-        # create a configuration with valid parameters
+        """create a configuration with valid parameters from config."""
         expected_configs = self.expected_default_datastore_configs()
         values = json.dumps(expected_configs.get('valid_values'))
         expected_values = json.loads(values)
@@ -281,12 +282,12 @@ class CreateConfigurations(ConfigurationsTestBase):
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 200)
         with TypeCheck('Configuration', result) as configuration:
-            configuration.has_field('name', six.string_types)
-            configuration.has_field('description', six.string_types)
+            configuration.has_field('name', str)
+            configuration.has_field('description', str)
             configuration.has_field('values', dict)
-            configuration.has_field('datastore_name', six.string_types)
-            configuration.has_field('datastore_version_id', six.text_type)
-            configuration.has_field('datastore_version_name', six.string_types)
+            configuration.has_field('datastore_name', str)
+            configuration.has_field('datastore_version_id', str)
+            configuration.has_field('datastore_version_name', str)
         global configuration_info
         configuration_info = result
         assert_equal(configuration_info.name, CONFIG_NAME)
@@ -295,6 +296,7 @@ class CreateConfigurations(ConfigurationsTestBase):
 
     @test(runs_after=[test_valid_configurations_create])
     def test_appending_to_existing_configuration(self):
+        """test_appending_to_existing_configuration"""
         # test being able to update and insert new parameter name and values
         # to an existing configuration
         expected_configs = self.expected_default_datastore_configs()
@@ -308,13 +310,13 @@ class CreateConfigurations(ConfigurationsTestBase):
         assert_equal(resp.status, 200)
 
 
-@test(runs_after=[CreateConfigurations],
-      groups=[GROUP, GROUP_CONFIG_DEFINE])
+@test(depends_on_classes=[CreateConfigurations],
+      groups=[tests.DBAAS_API_CONFIGURATIONS])
 class AfterConfigurationsCreation(ConfigurationsTestBase):
 
     @test
     def test_assign_configuration_to_invalid_instance(self):
-        # test assigning to an instance that does not exist
+        """test assigning to an instance that does not exist"""
         invalid_id = "invalid-inst-id"
         try:
             instance_info.dbaas.instances.modify(invalid_id,
@@ -325,7 +327,7 @@ class AfterConfigurationsCreation(ConfigurationsTestBase):
 
     @test
     def test_assign_configuration_to_valid_instance(self):
-        # test assigning a configuration to an instance
+        """test assigning a configuration to an instance"""
         print("instance_info.id: %s" % instance_info.id)
         print("configuration_info: %s" % configuration_info)
         print("configuration_info.id: %s" % configuration_info.id)
@@ -335,36 +337,9 @@ class AfterConfigurationsCreation(ConfigurationsTestBase):
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 202)
 
-    @test
-    def test_assign_name_to_instance_using_patch(self):
-        # test assigning a name to an instance
-        new_name = 'new_name_1'
-        report = CONFIG.get_report()
-        report.log("instance_info.id: %s" % instance_info.id)
-        report.log("instance name:%s" % instance_info.name)
-        report.log("instance new name:%s" % new_name)
-        instance_info.dbaas.instances.edit(instance_info.id, name=new_name)
-        assert_equal(202, instance_info.dbaas.last_http_code)
-        check = instance_info.dbaas.instances.get(instance_info.id)
-        assert_equal(200, instance_info.dbaas.last_http_code)
-        assert_equal(check.name, new_name)
-        # Restore instance name
-        instance_info.dbaas.instances.edit(instance_info.id,
-                                           name=instance_info.name)
-        assert_equal(202, instance_info.dbaas.last_http_code)
-
-    @test
-    def test_assign_configuration_to_invalid_instance_using_patch(self):
-        # test assign config group to an invalid instance
-        invalid_id = "invalid-inst-id"
-        assert_raises(exceptions.NotFound,
-                      instance_info.dbaas.instances.edit,
-                      invalid_id, configuration=configuration_info.id)
-
     @test(depends_on=[test_assign_configuration_to_valid_instance])
     def test_assign_configuration_to_instance_with_config(self):
-        # test assigning a configuration to an instance that
-        # already has an assigned configuration
+        """test assigning a configuration to an instance conflicts"""
         config_id = configuration_info.id
         assert_raises(exceptions.BadRequest,
                       instance_info.dbaas.instances.modify, instance_info.id,
@@ -373,17 +348,18 @@ class AfterConfigurationsCreation(ConfigurationsTestBase):
     @test(depends_on=[test_assign_configuration_to_valid_instance])
     @time_out(30)
     def test_get_configuration_details_from_instance_validation(self):
-        # validate that the configuration was applied correctly to the instance
+        """validate the configuration after attaching"""
         print("instance_info.id: %s" % instance_info.id)
         inst = instance_info.dbaas.instances.get(instance_info.id)
         configuration_id = inst.configuration['id']
         print("configuration_info: %s" % configuration_id)
-        assert_not_equal(None, inst.configuration['id'])
+        assert_not_equal(None, configuration_id)
         _test_configuration_is_applied_to_instance(instance_info,
                                                    configuration_id)
 
+    @test(depends_on=[test_get_configuration_details_from_instance_validation])
     def test_configurations_get(self):
-        # test that the instance shows up on the assigned configuration
+        """test that the instance shows up on the assigned configuration"""
         result = instance_info.dbaas.configurations.get(configuration_info.id)
         assert_equal(configuration_info.id, result.id)
         assert_equal(configuration_info.name, result.name)
@@ -391,12 +367,12 @@ class AfterConfigurationsCreation(ConfigurationsTestBase):
 
         # check the result field types
         with TypeCheck("configuration", result) as check:
-            check.has_field("id", six.string_types)
-            check.has_field("name", six.string_types)
-            check.has_field("description", six.string_types)
+            check.has_field("id", str)
+            check.has_field("name", str)
+            check.has_field("description", str)
             check.has_field("values", dict)
-            check.has_field("created", six.string_types)
-            check.has_field("updated", six.string_types)
+            check.has_field("created", str)
+            check.has_field("updated", str)
             check.has_field("instance_count", int)
 
         print(result.values)
@@ -426,7 +402,7 @@ class AfterConfigurationsCreation(ConfigurationsTestBase):
                 if param.type == 'integer':
                     check.has_element(item_key, int)
                 if param.type == 'string':
-                    check.has_element(item_key, six.string_types)
+                    check.has_element(item_key, str)
                 if param.type == 'boolean':
                     check.has_element(item_key, bool)
 
@@ -447,8 +423,8 @@ class AfterConfigurationsCreation(ConfigurationsTestBase):
                       configuration_info.id)
 
 
-@test(runs_after=[AfterConfigurationsCreation],
-      groups=[GROUP, GROUP_CONFIG_DEFINE])
+@test(depends_on_classes=[AfterConfigurationsCreation],
+      groups=[tests.DBAAS_API_CONFIGURATIONS])
 class ListConfigurations(ConfigurationsTestBase):
 
     @test
@@ -457,12 +433,12 @@ class ListConfigurations(ConfigurationsTestBase):
         result = instance_info.dbaas.configurations.list()
         for conf in result:
             with TypeCheck("Configuration", conf) as check:
-                check.has_field('id', six.string_types)
-                check.has_field('name', six.string_types)
-                check.has_field('description', six.string_types)
-                check.has_field('datastore_version_id', six.string_types)
-                check.has_field('datastore_version_name', six.string_types)
-                check.has_field('datastore_name', six.string_types)
+                check.has_field('id', str)
+                check.has_field('name', str)
+                check.has_field('description', str)
+                check.has_field('datastore_version_id', str)
+                check.has_field('datastore_version_name', str)
+                check.has_field('datastore_name', str)
 
         exists = [config for config in result if
                   config.id == configuration_info.id]
@@ -494,8 +470,7 @@ class ListConfigurations(ConfigurationsTestBase):
 
     @test
     def test_changing_configuration_with_nondynamic_parameter(self):
-        # test that changing a non-dynamic parameter is applied to instance
-        # and show that the instance requires a restart
+        """test_changing_configuration_with_nondynamic_parameter"""
         expected_configs = self.expected_default_datastore_configs()
         values = json.dumps(expected_configs.get('nondynamic_parameter'))
         instance_info.dbaas.configurations.update(configuration_info.id,
@@ -510,10 +485,11 @@ class ListConfigurations(ConfigurationsTestBase):
     @test(depends_on=[test_changing_configuration_with_nondynamic_parameter])
     @time_out(20)
     def test_waiting_for_instance_in_restart_required(self):
+        """test_waiting_for_instance_in_restart_required"""
         def result_is_not_active():
             instance = instance_info.dbaas.instances.get(
                 instance_info.id)
-            if instance.status == "ACTIVE":
+            if instance.status in CONFIG.running_status:
                 return False
             else:
                 return True
@@ -522,12 +498,11 @@ class ListConfigurations(ConfigurationsTestBase):
         instance = instance_info.dbaas.instances.get(instance_info.id)
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 200)
-        print(instance.status)
         assert_equal('RESTART_REQUIRED', instance.status)
 
     @test(depends_on=[test_waiting_for_instance_in_restart_required])
     def test_restart_service_should_return_active(self):
-        # test that after restarting the instance it becomes active
+        """test_restart_service_should_return_active"""
         instance_info.dbaas.instances.restart(instance_info.id)
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 202)
@@ -535,17 +510,17 @@ class ListConfigurations(ConfigurationsTestBase):
         def result_is_active():
             instance = instance_info.dbaas.instances.get(
                 instance_info.id)
-            if instance.status == "ACTIVE":
+            if instance.status in CONFIG.running_status:
                 return True
             else:
-                assert_equal("REBOOT", instance.status)
+                assert_true(instance.status in ['REBOOT', 'SHUTDOWN'])
                 return False
         poll_until(result_is_active)
 
     @test(depends_on=[test_restart_service_should_return_active])
     @time_out(30)
     def test_get_configuration_details_from_instance_validation(self):
-        # validate that the configuraiton was applied correctly to the instance
+        """test_get_configuration_details_from_instance_validation"""
         inst = instance_info.dbaas.instances.get(instance_info.id)
         configuration_id = inst.configuration['id']
         assert_not_equal(None, inst.configuration['id'])
@@ -565,13 +540,13 @@ class ListConfigurations(ConfigurationsTestBase):
         assert_equal(list_config[0].updated, details_config.updated)
 
 
-@test(runs_after=[ListConfigurations],
-      groups=[GROUP, GROUP_CONFIG_DEFINE])
+@test(depends_on_classes=[ListConfigurations],
+      groups=[tests.DBAAS_API_CONFIGURATIONS])
 class StartInstanceWithConfiguration(ConfigurationsTestBase):
 
     @test
     def test_start_instance_with_configuration(self):
-        # test that a new instance will apply the configuration on create
+        """test that a new instance will apply the configuration on create"""
         global configuration_instance
         databases = []
         databases.append({"name": "firstdbconfig", "character_set": "latin2",
@@ -608,19 +583,18 @@ class StartInstanceWithConfiguration(ConfigurationsTestBase):
 
 
 @test(depends_on_classes=[StartInstanceWithConfiguration],
-      runs_after_groups=['dbaas.api.backups'],
-      groups=[GROUP])
+      groups=[tests.DBAAS_API_CONFIGURATIONS])
 class WaitForConfigurationInstanceToFinish(ConfigurationsTestBase):
 
     @test
     @time_out(TIMEOUT_INSTANCE_CREATE)
     def test_instance_with_configuration_active(self):
-        # wait for the instance to become active
+        """wait for the instance created with configuration"""
 
         def result_is_active():
             instance = instance_info.dbaas.instances.get(
                 configuration_instance.id)
-            if instance.status == "ACTIVE":
+            if instance.status in CONFIG.running_status:
                 return True
             else:
                 assert_equal("BUILD", instance.status)
@@ -631,15 +605,16 @@ class WaitForConfigurationInstanceToFinish(ConfigurationsTestBase):
     @test(depends_on=[test_instance_with_configuration_active])
     @time_out(30)
     def test_get_configuration_details_from_instance_validation(self):
-        # validate that the configuration was applied correctly to the instance
+        """Test configuration is applied correctly to the instance."""
         inst = instance_info.dbaas.instances.get(configuration_instance.id)
         configuration_id = inst.configuration['id']
-        assert_not_equal(None, inst.configuration['id'])
+        assert_not_equal(None, configuration_id)
         _test_configuration_is_applied_to_instance(configuration_instance,
                                                    configuration_id)
 
 
-@test(runs_after=[WaitForConfigurationInstanceToFinish], groups=[GROUP])
+@test(depends_on=[WaitForConfigurationInstanceToFinish],
+      groups=[tests.DBAAS_API_CONFIGURATIONS])
 class DeleteConfigurations(ConfigurationsTestBase):
 
     @before_class
@@ -653,23 +628,24 @@ class DeleteConfigurations(ConfigurationsTestBase):
         resp, body = instance_info.dbaas.client.last_response
         print(resp)
         print(body)
-        self.config_parameter_dict = json.loads(body)
+        self.config_parameter_dict = json.loads(body.decode())
 
     @after_class(always_run=True)
     def tearDown(self):
         # need to "undelete" the parameter that was deleted from the mgmt call
-        ds = instance_info.dbaas_datastore
-        ds_v = instance_info.dbaas_datastore_version
-        version = instance_info.dbaas.datastore_versions.get(
-            ds, ds_v)
-        client = instance_info.dbaas_admin.mgmt_configs
-        print(self.config_parameter_dict)
-        client.create(version.id,
-                      self.config_parameter_dict['name'],
-                      self.config_parameter_dict['restart_required'],
-                      self.config_parameter_dict['type'],
-                      self.config_parameter_dict['max'],
-                      self.config_parameter_dict['min'])
+        if instance_info.dbaas:
+            ds = instance_info.dbaas_datastore
+            ds_v = instance_info.dbaas_datastore_version
+            version = instance_info.dbaas.datastore_versions.get(
+                ds, ds_v)
+            client = instance_info.dbaas_admin.mgmt_configs
+            print(self.config_parameter_dict)
+            client.create(version.id,
+                          self.config_parameter_dict['name'],
+                          self.config_parameter_dict['restart_required'],
+                          self.config_parameter_dict['type'],
+                          self.config_parameter_dict['max'],
+                          self.config_parameter_dict['min'])
 
     @test
     def test_delete_invalid_configuration_not_found(self):
@@ -709,15 +685,14 @@ class DeleteConfigurations(ConfigurationsTestBase):
     @test(depends_on=[test_unable_delete_instance_configurations])
     @time_out(30)
     def test_unassign_configuration_from_instances(self):
-        # test to unassign configuration from instance
-        instance_info.dbaas.instances.modify(configuration_instance.id,
-                                             configuration="")
+        """test to unassign configuration from instance"""
+        instance_info.dbaas.instances.update(configuration_instance.id,
+                                             remove_configuration=True)
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 202)
-        instance_info.dbaas.instances.get(configuration_instance.id)
-        # test that config group is not removed
-        instance_info.dbaas.instances.modify(instance_info.id,
-                                             configuration=None)
+
+        instance_info.dbaas.instances.update(instance_info.id,
+                                             remove_configuration=True)
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 202)
         instance_info.dbaas.instances.get(instance_info.id)
@@ -728,10 +703,12 @@ class DeleteConfigurations(ConfigurationsTestBase):
                 return False
             else:
                 return True
+
         inst_info = instance_info
         poll_until(result_has_no_configuration)
         inst_info = configuration_instance
         poll_until(result_has_no_configuration)
+
         instance = instance_info.dbaas.instances.get(instance_info.id)
         assert_equal('RESTART_REQUIRED', instance.status)
 
@@ -745,7 +722,7 @@ class DeleteConfigurations(ConfigurationsTestBase):
 
     @test(depends_on=[test_assign_in_wrong_state])
     def test_no_instances_on_configuration(self):
-        # test there is no configuration on the instance after unassigning
+        """test_no_instances_on_configuration"""
         result = instance_info.dbaas.configurations.get(configuration_info.id)
         assert_equal(configuration_info.id, result.id)
         assert_equal(configuration_info.name, result.name)
@@ -756,29 +733,8 @@ class DeleteConfigurations(ConfigurationsTestBase):
 
     @test(depends_on=[test_unassign_configuration_from_instances])
     @time_out(120)
-    def test_restart_service_after_unassign_return_active(self):
-        def result_is_not_active():
-            instance = instance_info.dbaas.instances.get(
-                instance_info.id)
-            if instance.status == "ACTIVE":
-                return False
-            else:
-                return True
-        poll_until(result_is_not_active)
-
-        config = instance_info.dbaas.configurations.list()
-        print(config)
-        instance = instance_info.dbaas.instances.get(instance_info.id)
-        print(instance.__dict__)
-        resp, body = instance_info.dbaas.client.last_response
-        assert_equal(resp.status, 200)
-        print(instance.status)
-        assert_equal('RESTART_REQUIRED', instance.status)
-
-    @test(depends_on=[test_restart_service_after_unassign_return_active])
-    @time_out(120)
     def test_restart_service_should_return_active(self):
-        # test that after restarting the instance it becomes active
+        """test that after restarting the instance it becomes active"""
         instance_info.dbaas.instances.restart(instance_info.id)
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 202)
@@ -786,7 +742,7 @@ class DeleteConfigurations(ConfigurationsTestBase):
         def result_is_active():
             instance = instance_info.dbaas.instances.get(
                 instance_info.id)
-            if instance.status == "ACTIVE":
+            if instance.status in CONFIG.running_status:
                 return True
             else:
                 assert_equal("REBOOT", instance.status)
@@ -795,7 +751,7 @@ class DeleteConfigurations(ConfigurationsTestBase):
 
     @test(depends_on=[test_restart_service_should_return_active])
     def test_assign_config_and_name_to_instance_using_patch(self):
-        # test assigning a configuration and name to an instance
+        """test_assign_config_and_name_to_instance_using_patch"""
         new_name = 'new_name'
         report = CONFIG.get_report()
         report.log("instance_info.id: %s" % instance_info.id)
@@ -805,17 +761,17 @@ class DeleteConfigurations(ConfigurationsTestBase):
         report.log("instance new name:%s" % new_name)
         saved_name = instance_info.name
         config_id = configuration_info.id
-        instance_info.dbaas.instances.edit(instance_info.id,
-                                           configuration=config_id,
-                                           name=new_name)
+        instance_info.dbaas.instances.update(instance_info.id,
+                                             configuration=config_id,
+                                             name=new_name)
         assert_equal(202, instance_info.dbaas.last_http_code)
         check = instance_info.dbaas.instances.get(instance_info.id)
         assert_equal(200, instance_info.dbaas.last_http_code)
         assert_equal(check.name, new_name)
 
         # restore instance name
-        instance_info.dbaas.instances.edit(instance_info.id,
-                                           name=saved_name)
+        instance_info.dbaas.instances.update(instance_info.id,
+                                             name=saved_name)
         assert_equal(202, instance_info.dbaas.last_http_code)
 
         instance = instance_info.dbaas.instances.get(instance_info.id)
@@ -828,7 +784,7 @@ class DeleteConfigurations(ConfigurationsTestBase):
         def result_is_active():
             instance = instance_info.dbaas.instances.get(
                 instance_info.id)
-            if instance.status == "ACTIVE":
+            if instance.status in CONFIG.running_status:
                 return True
             else:
                 assert_equal("REBOOT", instance.status)
@@ -838,14 +794,14 @@ class DeleteConfigurations(ConfigurationsTestBase):
         # already has an assigned configuration with patch
         config_id = configuration_info.id
         assert_raises(exceptions.BadRequest,
-                      instance_info.dbaas.instances.edit,
+                      instance_info.dbaas.instances.update,
                       instance_info.id, configuration=config_id)
 
     @test(runs_after=[test_assign_config_and_name_to_instance_using_patch])
     def test_unassign_configuration_after_patch(self):
-        # remove the configuration from the instance
-        instance_info.dbaas.instances.edit(instance_info.id,
-                                           remove_configuration=True)
+        """Remove the configuration from the instance"""
+        instance_info.dbaas.instances.update(instance_info.id,
+                                             remove_configuration=True)
         assert_equal(202, instance_info.dbaas.last_http_code)
         instance = instance_info.dbaas.instances.get(instance_info.id)
         assert_equal('RESTART_REQUIRED', instance.status)
@@ -857,11 +813,12 @@ class DeleteConfigurations(ConfigurationsTestBase):
         def result_is_active():
             instance = instance_info.dbaas.instances.get(
                 instance_info.id)
-            if instance.status == "ACTIVE":
+            if instance.status in CONFIG.running_status:
                 return True
             else:
                 assert_equal("REBOOT", instance.status)
                 return False
+
         poll_until(result_is_active)
         result = instance_info.dbaas.configurations.get(configuration_info.id)
         assert_equal(result.instance_count, 0)
@@ -871,16 +828,15 @@ class DeleteConfigurations(ConfigurationsTestBase):
         # test unassign config group from an invalid instance
         invalid_id = "invalid-inst-id"
         try:
-            instance_info.dbaas.instances.edit(invalid_id,
-                                               remove_configuration=True)
+            instance_info.dbaas.instances.update(invalid_id,
+                                                 remove_configuration=True)
         except exceptions.NotFound:
             resp, body = instance_info.dbaas.client.last_response
             assert_equal(resp.status, 404)
 
     @test(runs_after=[test_unassign_configuration_after_patch])
     def test_delete_unassigned_configuration(self):
-        # test that we can delete the configuration after no instances are
-        # assigned to it any longer
+        """test_delete_unassigned_configuration"""
         instance_info.dbaas.configurations.delete(configuration_info.id)
         resp, body = instance_info.dbaas.client.last_response
         assert_equal(resp.status, 202)
@@ -888,8 +844,7 @@ class DeleteConfigurations(ConfigurationsTestBase):
     @test(depends_on=[test_delete_unassigned_configuration])
     @time_out(TIMEOUT_INSTANCE_DELETE)
     def test_delete_configuration_instance(self):
-        # test that we can delete the instance even though there is a
-        # configuration applied to the instance
+        """test_delete_configuration_instance"""
         instance_info.dbaas.instances.delete(configuration_instance.id)
         assert_equal(202, instance_info.dbaas.last_http_code)
 
