@@ -13,19 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
+import uuid
+
+import eventlet
+
 from novaclient import exceptions as nova_exceptions
 from oslo_log import log as logging
 
 from trove.common.exception import PollTimeOut
-from trove.common.i18n import _
-from trove.common import instance as rd_instance
+from trove.instance import service_status as srvstatus
 from trove.tests.fakes.common import authorize
 
-import collections
-import eventlet
-import uuid
-
-
 LOG = logging.getLogger(__name__)
 FAKE_HOSTS = ["fake_host_1", "fake_host_2"]
 
@@ -102,7 +101,7 @@ class FakeServer(object):
     next_local_id = 0
 
     def __init__(self, parent, owner, id, name, image_id, flavor_ref,
-                 block_device_mapping, volumes):
+                 volumes, key_name):
         self.owner = owner  # This is a context.
         self.id = id
         self.parent = parent
@@ -126,15 +125,19 @@ class FakeServer(object):
         setattr(self, 'OS-EXT-AZ:availability_zone', 'nova')
 
         self._info = {'os:volumes': info_vols}
+        self.key_name = key_name
 
     @property
     def addresses(self):
-        return {"private": [{"addr": "123.123.123.123"}]}
+        return [{
+            "address": "123.123.123.123",
+            'type': 'private',
+            'network': 'net-id'}]
 
     def confirm_resize(self):
         if self.status != "VERIFY_RESIZE":
             raise RuntimeError("Not in resize confirm mode.")
-        self._current_status = "ACTIVE"
+        self._current_status = "HEALTHY"
 
     def revert_resize(self):
         if self.status != "VERIFY_RESIZE":
@@ -143,13 +146,13 @@ class FakeServer(object):
         self.old_host = None
         self.flavor_ref = self.old_flavor_ref
         self.old_flavor_ref = None
-        self._current_status = "ACTIVE"
+        self._current_status = "HEALTHY"
 
     def reboot(self):
-        LOG.debug("Rebooting server %s" % (self.id))
+        LOG.debug("Rebooting server %s", self.id)
 
         def set_to_active():
-            self._current_status = "ACTIVE"
+            self._current_status = "HEALTHY"
             self.parent.schedule_simulate_running_server(self.id, 1.5)
 
         self._current_status = "REBOOT"
@@ -204,7 +207,7 @@ class FakeServer(object):
 
         def set_flavor():
             if self.name.endswith("_RESIZE_ERROR"):
-                self._current_status = "ACTIVE"
+                self._current_status = "HEALTHY"
                 return
             if new_flavor_id is None:
                 # Migrations are flavorless flavor resizes.
@@ -264,33 +267,16 @@ class FakeServers(object):
         """Can this FakeServers, with its context, see some resource?"""
         server = self.db[id]
         return (self.context.is_admin or
-                server.owner.tenant == self.context.tenant)
+                server.owner.tenant == self.context.project_id)
 
     def create(self, name, image_id, flavor_ref, files=None, userdata=None,
-               block_device_mapping=None, volume=None, security_groups=None,
+               block_device_mapping_v2=None, security_groups=None,
                availability_zone=None, nics=None, config_drive=False,
-               scheduler_hints=None):
+               scheduler_hints=None, key_name=None):
         id = "FAKE_%s" % uuid.uuid4()
-        if volume:
-            volume = self.volumes.create(volume['size'], volume['name'],
-                                         volume['description'])
-            while volume.status == "BUILD":
-                eventlet.sleep(0.1)
-            if volume.status != "available":
-                LOG.info(_("volume status = %s") % volume.status)
-                raise nova_exceptions.ClientException("Volume was bad!")
-            mapping = "%s::%s:%s" % (volume.id, volume.size, 1)
-            block_device_mapping = {'vdb': mapping}
-            volumes = [volume]
-            LOG.debug("Fake Volume Create %(volumeid)s with "
-                      "status %(volumestatus)s" %
-                      {'volumeid': volume.id, 'volumestatus': volume.status})
-        else:
-            volumes = self._get_volumes_from_bdm(block_device_mapping)
-            for volume in volumes:
-                volume.schedule_status('in-use', 1)
+        volumes = self._get_volumes_from_bdm_v2(block_device_mapping_v2)
         server = FakeServer(self, self.context, id, name, image_id, flavor_ref,
-                            block_device_mapping, volumes)
+                            volumes, key_name)
         self.db[id] = server
         if name.endswith('SERVER_ERROR'):
             raise nova_exceptions.ClientException("Fake server create error.")
@@ -299,37 +285,25 @@ class FakeServers(object):
             raise nova_exceptions.ClientException("The requested availability "
                                                   "zone is not available.")
 
-        if nics:
-            if 'port-id' in nics[0] and nics[0]['port-id'] == "UNKNOWN":
-                raise nova_exceptions.ClientException("The requested "
-                                                      "port-id is not "
-                                                      "available.")
-
-        server.schedule_status("ACTIVE", 1)
-        LOG.info("FAKE_SERVERS_DB : %s" % str(FAKE_SERVERS_DB))
+        server.schedule_status("HEALTHY", 1)
+        LOG.info("FAKE_SERVERS_DB : %s", str(FAKE_SERVERS_DB))
         return server
 
-    def _get_volumes_from_bdm(self, block_device_mapping):
+    def _get_volumes_from_bdm_v2(self, block_device_mapping_v2):
         volumes = []
-        if block_device_mapping is not None:
-            # block_device_mapping is a dictionary, where the key is the
-            # device name on the compute instance and the mapping info is a
-            # set of fields in a string, separated by colons.
-            # For each device, find the volume, and record the mapping info
-            # to another fake object and attach it to the volume
-            # so that the fake API can later retrieve this.
-            for device in block_device_mapping:
-                mapping = block_device_mapping[device]
-                (id, _type, size, delete_on_terminate) = mapping.split(":")
-                volume = self.volumes.get(id)
-                volume.mapping = FakeBlockDeviceMappingInfo(
-                    id, device, _type, size, delete_on_terminate)
+        if block_device_mapping_v2 is not None:
+            # block_device_mapping_v2 is an array of dicts. Every dict is
+            # a volume attachment, which contains "uuid", "source_type",
+            # "destination_type", "device_name", "volume_size" and
+            # "delete_on_termination".
+            for bdm in block_device_mapping_v2:
+                volume = self.volumes.get(bdm['uuid'])
                 volumes.append(volume)
         return volumes
 
     def get(self, id):
         if id not in self.db:
-            LOG.error(_("Couldn't find server id %(id)s, collection=%(db)s") %
+            LOG.error("Couldn't find server id %(id)s, collection=%(db)s",
                       {'id': id, 'db': self.db})
             raise nova_exceptions.NotFound(404, "Not found")
         else:
@@ -338,18 +312,12 @@ class FakeServers(object):
             else:
                 raise nova_exceptions.NotFound(404, "Bad permissions")
 
-    def get_server_volumes(self, server_id):
-        """Fake method we've added to grab servers from the volume."""
-        return [volume.mapping
-                for volume in self.get(server_id).volumes
-                if volume.mapping is not None]
-
     def list(self):
         return [v for (k, v) in self.db.items() if self.can_see(v.id)]
 
     def schedule_delete(self, id, time_from_now):
         def delete_server():
-            LOG.info(_("Simulated event ended, deleting server %s.") % id)
+            LOG.info("Simulated event ended, deleting server %s.", id)
             del self.db[id]
         eventlet.spawn_after(time_from_now, delete_server)
 
@@ -359,9 +327,9 @@ class FakeServers(object):
 
         def set_server_running():
             instance = DBInstance.find_by(compute_instance_id=id)
-            LOG.debug("Setting server %s to running" % instance.id)
+            LOG.debug("Setting server %s to running", instance.id)
             status = InstanceServiceStatus.find_by(instance_id=instance.id)
-            status.status = rd_instance.ServiceStatuses.RUNNING
+            status.status = srvstatus.ServiceStatuses.RUNNING
             status.save()
         eventlet.spawn_after(time_from_now, set_server_running)
 
@@ -391,30 +359,10 @@ class FakeRdServers(object):
         return [FakeRdServer(server) for server in self.servers.list()]
 
 
-class FakeServerVolumes(object):
-
-    def __init__(self, context):
-        self.context = context
-
-    def get_server_volumes(self, server_id):
-        class ServerVolumes(object):
-            def __init__(self, block_device_mapping):
-                LOG.debug("block_device_mapping = %s" %
-                          block_device_mapping)
-                device = block_device_mapping['vdb']
-                (self.volumeId,
-                    self.type,
-                    self.size,
-                    self.delete_on_terminate) = device.split(":")
-        fake_servers = FakeServers(self.context, FLAVORS)
-        server = fake_servers.get(server_id)
-        return [ServerVolumes(server.block_device_mapping)]
-
-
 class FakeVolume(object):
 
     def __init__(self, parent, owner, id, size, name,
-                 description, volume_type):
+                 description, volume_type, availability_zone):
         self.attachments = []
         self.parent = parent
         self.owner = owner  # This is a context.
@@ -427,6 +375,7 @@ class FakeVolume(object):
         # point.
         self.device = "vdb"
         self.volume_type = volume_type
+        self.availability_zone = availability_zone
 
     def __repr__(self):
         msg = ("FakeVolume(id=%s, size=%s, name=%s, "
@@ -435,10 +384,6 @@ class FakeVolume(object):
                   self.description, self._current_status)
         return (msg % params)
 
-    @property
-    def availability_zone(self):
-        return "fake-availability-zone"
-
     @property
     def created_at(self):
         return "2001-01-01-12:30:30"
@@ -488,11 +433,11 @@ class FakeVolumes(object):
         """Can this FakeVolumes, with its context, see some resource?"""
         server = self.db[id]
         return (self.context.is_admin or
-                server.owner.tenant == self.context.tenant)
+                server.owner.tenant == self.context.project_id)
 
     def get(self, id):
         if id not in self.db:
-            LOG.error(_("Couldn't find volume id %(id)s, collection=%(db)s") %
+            LOG.error("Couldn't find volume id %(id)s, collection=%(db)s",
                       {'id': id, 'db': self.db})
             raise nova_exceptions.NotFound(404, "Not found")
         else:
@@ -501,10 +446,11 @@ class FakeVolumes(object):
             else:
                 raise nova_exceptions.NotFound(404, "Bad permissions")
 
-    def create(self, size, name=None, description=None, volume_type=None):
+    def create(self, size, name=None, description=None, volume_type=None,
+               availability_zone=None):
         id = "FAKE_VOL_%s" % uuid.uuid4()
         volume = FakeVolume(self, self.context, id, size, name,
-                            description, volume_type)
+                            description, volume_type, availability_zone)
         self.db[id] = volume
         if size == 9:
             volume.schedule_status("error", 2)
@@ -513,16 +459,16 @@ class FakeVolumes(object):
         else:
             volume.schedule_status("available", 2)
         LOG.debug("Fake volume created %(volumeid)s with "
-                  "status %(volumestatus)s" %
+                  "status %(volumestatus)s",
                   {'volumeid': volume.id, 'volumestatus': volume.status})
-        LOG.info("FAKE_VOLUMES_DB : %s" % FAKE_VOLUMES_DB)
+        LOG.info("FAKE_VOLUMES_DB : %s", FAKE_VOLUMES_DB)
         return volume
 
     def list(self, detailed=True):
         return [self.db[key] for key in self.db]
 
     def extend(self, volume_id, new_size):
-        LOG.debug("Resize volume id (%(volumeid)s) to size (%(size)s)" %
+        LOG.debug("Resize volume id (%(volumeid)s) to size (%(size)s)",
                   {'volumeid': volume_id, 'size': new_size})
         volume = self.get(volume_id)
 
@@ -634,7 +580,7 @@ class FakeHost(object):
                 'status': server.status
             })
             if (str(server.flavor_ref).startswith('http:') or
-               str(server.flavor_ref).startswith('https:')):
+                    str(server.flavor_ref).startswith('https:')):
                 flavor = FLAVORS.get_by_href(server.flavor_ref)
             else:
                 flavor = FLAVORS.get(server.flavor_ref)
@@ -848,9 +794,6 @@ class FakeClient(object):
         self.security_group_rules = FakeSecurityGroupRules(context)
         self.server_groups = FakeServerGroups(context)
 
-    def get_server_volumes(self, server_id):
-        return self.servers.get_server_volumes(server_id)
-
     def rescan_server_volume(self, server, volume_id):
         LOG.info("FAKE rescanning volume.")
 
@@ -870,13 +813,13 @@ def get_client_data(context):
     return CLIENT_DATA[context]
 
 
-def fake_create_nova_client(context):
+def fake_create_nova_client(context, region_name=None):
     return get_client_data(context)['nova']
 
 
-def fake_create_nova_volume_client(context):
+def fake_create_nova_volume_client(context, region_name=None):
     return get_client_data(context)['volume']
 
 
-def fake_create_cinder_client(context):
+def fake_create_cinder_client(context, region_name=None):
     return get_client_data(context)['volume']
