@@ -22,6 +22,8 @@ from trove.guestagent.common import operating_system
 from trove.guestagent import guest_log
 from trove.tests.config import CONFIG
 from trove.tests.scenario.helpers.test_helper import DataType
+from trove.tests.scenario import runners
+from trove.tests.scenario.runners.test_runners import SkipKnownBug
 from trove.tests.scenario.runners.test_runners import TestRunner
 
 
@@ -71,6 +73,7 @@ class GuestLogRunner(TestRunner):
         log_list = list(client.instances.log_list(self.instance_info.id))
         log_names = list(ll.name for ll in log_list)
         self.assert_list_elements_equal(expected_list, log_names)
+        self.register_debug_inst_ids(self.instance_info.id)
 
     def run_test_admin_log_list(self):
         self.assert_log_list(self.admin_client,
@@ -78,8 +81,9 @@ class GuestLogRunner(TestRunner):
 
     def run_test_log_show(self):
         log_pending = self._set_zero_or_none()
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_show(self.auth_client,
-                             self._get_exposed_user_log_name(),
+                             log_name,
                              expected_published=0,
                              expected_pending=log_pending)
 
@@ -97,22 +101,25 @@ class GuestLogRunner(TestRunner):
                         expected_http_code=200,
                         expected_type=guest_log.LogType.USER.name,
                         expected_status=guest_log.LogStatus.Disabled.name,
-                        expected_published=None, expected_pending=None):
+                        expected_published=None, expected_pending=None,
+                        is_admin=False):
         self.report.log("Executing log_show for log '%s'" % log_name)
         log_details = client.instances.log_show(
             self.instance_info.id, log_name)
-        self.assert_client_code(expected_http_code, client=client)
+        self.assert_client_code(client, expected_http_code)
         self.assert_log_details(
             log_details, log_name,
             expected_type=expected_type,
             expected_status=expected_status,
             expected_published=expected_published,
-            expected_pending=expected_pending)
+            expected_pending=expected_pending,
+            is_admin=is_admin)
 
     def assert_log_details(self, log_details, expected_log_name,
                            expected_type=guest_log.LogType.USER.name,
                            expected_status=guest_log.LogStatus.Disabled.name,
-                           expected_published=None, expected_pending=None):
+                           expected_published=None, expected_pending=None,
+                           is_admin=False):
         """Check that the action generates the proper response data.
         For log_published and log_pending, setting the value to 'None'
         will skip that check (useful when using an existing instance,
@@ -129,8 +136,11 @@ class GuestLogRunner(TestRunner):
         self.assert_equal(expected_type, log_details.type,
                           "Wrong log type for '%s' log" % expected_log_name)
         current_status = log_details.status.replace(' ', '_')
-        self.assert_equal(expected_status, current_status,
-                          "Wrong log status for '%s' log" % expected_log_name)
+        if not isinstance(expected_status, list):
+            expected_status = [expected_status]
+        self.assert_is_sublist([current_status], expected_status,
+                               "Wrong log status for '%s' log" %
+                               expected_log_name)
         if expected_published is None:
             pass
         elif expected_published == 0:
@@ -155,18 +165,23 @@ class GuestLogRunner(TestRunner):
                              "expected %d, got %d" %
                              (expected_log_name, expected_pending,
                               log_details.pending))
+
         container = self.container
         prefix = self.prefix_pattern % {
             'instance_id': self.instance_info.id,
             'datastore': CONFIG.dbaas_datastore,
             'log': expected_log_name}
         metafile = prefix.rstrip('/') + '_metafile'
+
         if expected_published == 0:
-            self.assert_storage_gone(container, prefix, metafile)
+            self.assert_storage_gone(container, prefix, metafile,
+                                     is_admin=is_admin)
             container = 'None'
             prefix = 'None'
         else:
-            self.assert_storage_exists(container, prefix, metafile)
+            self.assert_storage_exists(container, prefix, metafile,
+                                       is_admin=is_admin)
+
         self.assert_equal(container, log_details.container,
                           "Wrong log container for '%s' log" %
                           expected_log_name)
@@ -182,9 +197,9 @@ class GuestLogRunner(TestRunner):
                           expected_status=guest_log.LogStatus.Disabled.name,
                           expected_published=None, expected_pending=None):
         self.report.log("Executing log_enable for log '%s'" % log_name)
-        log_details = client.instances.log_enable(
-            self.instance_info.id, log_name)
-        self.assert_client_code(expected_http_code, client=client)
+        log_details = client.instances.log_action(
+            self.instance_info.id, log_name, enable=True)
+        self.assert_client_code(client, expected_http_code)
         self.assert_log_details(
             log_details, log_name,
             expected_type=expected_type,
@@ -199,9 +214,9 @@ class GuestLogRunner(TestRunner):
                            expected_published=None, expected_pending=None):
         self.report.log("Executing log_disable for log '%s' (discard: %s)" %
                         (log_name, discard))
-        log_details = client.instances.log_disable(
-            self.instance_info.id, log_name, discard=discard)
-        self.assert_client_code(expected_http_code, client=client)
+        log_details = client.instances.log_action(
+            self.instance_info.id, log_name, disable=True, discard=discard)
+        self.assert_client_code(client, expected_http_code)
         self.assert_log_details(
             log_details, log_name,
             expected_type=expected_type,
@@ -213,19 +228,22 @@ class GuestLogRunner(TestRunner):
                            expected_http_code=200,
                            expected_type=guest_log.LogType.USER.name,
                            expected_status=guest_log.LogStatus.Disabled.name,
-                           expected_published=None, expected_pending=None):
+                           expected_published=None, expected_pending=None,
+                           is_admin=False):
         self.report.log("Executing log_publish for log '%s' (disable: %s  "
                         "discard: %s)" %
                         (log_name, disable, discard))
-        log_details = client.instances.log_publish(
-            self.instance_info.id, log_name, disable=disable, discard=discard)
-        self.assert_client_code(expected_http_code, client=client)
+        log_details = client.instances.log_action(
+            self.instance_info.id, log_name, publish=True, disable=disable,
+            discard=discard)
+        self.assert_client_code(client, expected_http_code)
         self.assert_log_details(
             log_details, log_name,
             expected_type=expected_type,
             expected_status=expected_status,
             expected_published=expected_published,
-            expected_pending=expected_pending)
+            expected_pending=expected_pending,
+            is_admin=is_admin)
 
     def assert_log_discard(self, client, log_name,
                            expected_http_code=200,
@@ -233,9 +251,9 @@ class GuestLogRunner(TestRunner):
                            expected_status=guest_log.LogStatus.Disabled.name,
                            expected_published=None, expected_pending=None):
         self.report.log("Executing log_discard for log '%s'" % log_name)
-        log_details = client.instances.log_discard(
-            self.instance_info.id, log_name)
-        self.assert_client_code(expected_http_code, client=client)
+        log_details = client.instances.log_action(
+            self.instance_info.id, log_name, discard=True)
+        self.assert_client_code(client, expected_http_code)
         self.assert_log_details(
             log_details, log_name,
             expected_type=expected_type,
@@ -243,9 +261,14 @@ class GuestLogRunner(TestRunner):
             expected_published=expected_published,
             expected_pending=expected_pending)
 
-    def assert_storage_gone(self, container, prefix, metafile):
+    def assert_storage_gone(self, container, prefix, metafile, is_admin=False):
+        if is_admin:
+            swift_client = self.admin_swift_client
+        else:
+            swift_client = self.swift_client
+
         try:
-            headers, container_files = self.swift_client.get_container(
+            headers, container_files = swift_client.get_container(
                 container, prefix=prefix)
             self.assert_equal(0, len(container_files),
                               "Found files in %s/%s: %s" %
@@ -258,7 +281,7 @@ class GuestLogRunner(TestRunner):
             else:
                 raise
         try:
-            self.swift_client.get_object(container, metafile)
+            swift_client.get_object(container, metafile)
             self.fail("Found metafile after discard: %s" % metafile)
         except ClientException as ex:
             if ex.http_status == 404:
@@ -268,9 +291,15 @@ class GuestLogRunner(TestRunner):
             else:
                 raise
 
-    def assert_storage_exists(self, container, prefix, metafile):
+    def assert_storage_exists(self, container, prefix, metafile,
+                              is_admin=False):
+        if is_admin:
+            swift_client = self.admin_swift_client
+        else:
+            swift_client = self.swift_client
+
         try:
-            headers, container_files = self.swift_client.get_container(
+            headers, container_files = swift_client.get_container(
                 container, prefix=prefix)
             self.assert_true(len(container_files) > 0,
                              "No files found in %s/%s" %
@@ -281,7 +310,7 @@ class GuestLogRunner(TestRunner):
             else:
                 raise
         try:
-            self.swift_client.get_object(container, metafile)
+            swift_client.get_object(container, metafile)
         except ClientException as ex:
             if ex.http_status == 404:
                 self.fail("Missing metafile: %s" % metafile)
@@ -291,140 +320,146 @@ class GuestLogRunner(TestRunner):
     def run_test_log_enable_sys(self,
                                 expected_exception=exceptions.BadRequest,
                                 expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_enable_fails(
             self.admin_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def assert_log_enable_fails(self, client,
                                 expected_exception, expected_http_code,
                                 log_name):
-        self.assert_raises(expected_exception, None,
-                           client.instances.log_enable,
-                           self.instance_info.id, log_name)
-        # we may not be using the main client, so check explicitly here
-        self.assert_client_code(expected_http_code, client=client)
+        self.assert_raises(expected_exception, expected_http_code,
+                           client, client.instances.log_action,
+                           self.instance_info.id, log_name, enable=True)
 
     def run_test_log_disable_sys(self,
                                  expected_exception=exceptions.BadRequest,
                                  expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_disable_fails(
             self.admin_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def assert_log_disable_fails(self, client,
                                  expected_exception, expected_http_code,
                                  log_name, discard=None):
-        self.assert_raises(expected_exception, None,
-                           client.instances.log_disable,
-                           self.instance_info.id, log_name,
+        self.assert_raises(expected_exception, expected_http_code,
+                           client, client.instances.log_action,
+                           self.instance_info.id, log_name, disable=True,
                            discard=discard)
-        # we may not be using the main client, so check explicitly here
-        self.assert_client_code(expected_http_code, client=client)
 
     def run_test_log_show_unauth_user(self,
                                       expected_exception=exceptions.NotFound,
                                       expected_http_code=404):
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_show_fails(
             self.unauth_client,
             expected_exception, expected_http_code,
-            self._get_exposed_user_log_name())
+            log_name)
 
     def assert_log_show_fails(self, client,
                               expected_exception, expected_http_code,
                               log_name):
-        self.assert_raises(expected_exception, None,
-                           client.instances.log_show,
+        self.assert_raises(expected_exception, expected_http_code,
+                           client, client.instances.log_show,
                            self.instance_info.id, log_name)
-        # we may not be using the main client, so check explicitly here
-        self.assert_client_code(expected_http_code, client=client)
 
     def run_test_log_list_unauth_user(self,
                                       expected_exception=exceptions.NotFound,
                                       expected_http_code=404):
-        self.assert_raises(expected_exception, None,
-                           self.unauth_client.instances.log_list,
+        client = self.unauth_client
+        self.assert_raises(expected_exception, expected_http_code,
+                           client, client.instances.log_list,
                            self.instance_info.id)
-        # we're not using the main client, so check explicitly here
-        self.assert_client_code(expected_http_code, client=self.unauth_client)
 
-    def run_test_log_generator_unauth_user(self):
+    def run_test_log_generator_unauth_user(
+            self, expected_exception=exceptions.NotFound,
+            expected_http_code=404):
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_generator_unauth_user(
-            self.unauth_client, self._get_exposed_user_log_name())
+            self.unauth_client, log_name,
+            expected_exception, expected_http_code)
 
-    def assert_log_generator_unauth_user(self, client, log_name, publish=None):
-        try:
-            client.instances.log_generator(
-                self.instance_info.id, log_name, publish=publish)
-            raise("Client allowed unauthorized access to log_generator")
-        except Exception:
-            pass
+    def assert_log_generator_unauth_user(self, client, log_name,
+                                         expected_exception,
+                                         expected_http_code,
+                                         publish=None):
+        raise SkipKnownBug(runners.BUG_UNAUTH_TEST_WRONG)
+        # self.assert_raises(expected_exception, expected_http_code,
+        #                    client, client.instances.log_generator,
+        #                    self.instance_info.id, log_name, publish=publish)
 
-    def run_test_log_generator_publish_unauth_user(self):
+    def run_test_log_generator_publish_unauth_user(
+            self, expected_exception=exceptions.NotFound,
+            expected_http_code=404):
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_generator_unauth_user(
-            self.unauth_client, self._get_exposed_user_log_name(),
+            self.unauth_client, log_name,
+            expected_exception, expected_http_code,
             publish=True)
 
     def run_test_log_show_unexposed_user(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_show_fails(
             self.auth_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def run_test_log_enable_unexposed_user(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_enable_fails(
             self.auth_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def run_test_log_disable_unexposed_user(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_disable_fails(
             self.auth_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def run_test_log_publish_unexposed_user(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_publish_fails(
             self.auth_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def assert_log_publish_fails(self, client,
                                  expected_exception, expected_http_code,
                                  log_name,
                                  disable=None, discard=None):
-        self.assert_raises(expected_exception, None,
-                           client.instances.log_publish,
-                           self.instance_info.id, log_name,
+        self.assert_raises(expected_exception, expected_http_code,
+                           client, client.instances.log_action,
+                           self.instance_info.id, log_name, publish=True,
                            disable=disable, discard=discard)
-        # we may not be using the main client, so check explicitly here
-        self.assert_client_code(expected_http_code, client=client)
 
     def run_test_log_discard_unexposed_user(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_discard_fails(
             self.auth_client,
             expected_exception, expected_http_code,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def assert_log_discard_fails(self, client,
                                  expected_exception, expected_http_code,
                                  log_name):
-        self.assert_raises(expected_exception, None,
-                           client.instances.log_discard,
-                           self.instance_info.id, log_name)
-        # we may not be using the main client, so check explicitly here
-        self.assert_client_code(expected_http_code, client=client)
+        self.assert_raises(expected_exception, expected_http_code,
+                           client, client.instances.log_action,
+                           self.instance_info.id, log_name, discard=True)
 
     def run_test_log_enable_user(self):
         expected_status = guest_log.LogStatus.Ready.name
@@ -466,24 +501,24 @@ class GuestLogRunner(TestRunner):
             instance_id = self.instance_info.id
             # we need to wait until the heartbeat flips the instance
             # back into 'ACTIVE' before we issue the restart command
-            expected_states = ['RESTART_REQUIRED', 'ACTIVE']
-            self.assert_instance_action(instance_id, expected_states, None)
-            self.auth_client.instances.restart(instance_id)
-            self.assert_client_code(expected_http_code,
-                                    client=self.auth_client)
+            expected_states = ['RESTART_REQUIRED', 'HEALTHY']
+            self.assert_instance_action(instance_id, expected_states)
+            client = self.auth_client
+            client.instances.restart(instance_id)
+            self.assert_client_code(client, expected_http_code)
 
-    def run_test_wait_for_restart(self, expected_states=['REBOOT', 'ACTIVE']):
+    def run_test_wait_for_restart(self, expected_states=['REBOOT', 'HEALTHY']):
         if self.test_helper.log_enable_requires_restart():
-            self.assert_instance_action(self.instance_info.id,
-                                        expected_states, None)
+            self.assert_instance_action(self.instance_info.id, expected_states)
 
     def run_test_log_publish_user(self):
         for log_name in self._get_exposed_user_log_names():
             self.assert_log_publish(
                 self.auth_client,
                 log_name,
-                expected_status=guest_log.LogStatus.Published.name,
-                expected_published=1, expected_pending=0)
+                expected_status=[guest_log.LogStatus.Published.name,
+                                 guest_log.LogStatus.Partial.name],
+                expected_published=1, expected_pending=None)
 
     def run_test_add_data(self):
         self.test_helper.add_data(DataType.micro, self.get_instance_host())
@@ -494,11 +529,12 @@ class GuestLogRunner(TestRunner):
     def run_test_log_publish_again_user(self):
         for log_name in self._get_exposed_user_log_names():
             self.assert_log_publish(
-                self.admin_client,
+                self.auth_client,
                 log_name,
-                expected_status=guest_log.LogStatus.Published.name,
+                expected_status=[guest_log.LogStatus.Published.name,
+                                 guest_log.LogStatus.Partial.name],
                 expected_published=self._get_last_log_published(log_name),
-                expected_pending=0)
+                expected_pending=None)
 
     def run_test_log_generator_user(self):
         for log_name in self._get_exposed_user_log_names():
@@ -512,12 +548,19 @@ class GuestLogRunner(TestRunner):
                              swift_client=None):
         self.report.log("Executing log_generator for log '%s' (publish: %s)" %
                         (log_name, publish))
+
+        if publish:
+            client.instances.log_action(self.instance_info.id, log_name,
+                                        publish=True)
+
         log_gen = client.instances.log_generator(
             self.instance_info.id, log_name,
-            publish=publish, lines=lines, swift=swift_client)
+            lines=lines, swift=swift_client)
         log_contents = "".join([chunk for chunk in log_gen()])
+
         self.report.log("Returned %d lines for log '%s': %s" % (
             len(log_contents.splitlines()), log_name, log_contents))
+
         self._set_last_log_contents(log_name, log_contents)
         if expected_lines:
             self.assert_equal(expected_lines,
@@ -556,9 +599,10 @@ class GuestLogRunner(TestRunner):
         self.assert_log_publish(
             self.auth_client,
             log_name,
-            expected_status=guest_log.LogStatus.Published.name,
+            expected_status=[guest_log.LogStatus.Published.name,
+                             guest_log.LogStatus.Partial.name],
             expected_published=self._get_last_log_published(log_name),
-            expected_pending=0)
+            expected_pending=None)
         # Now get the full contents of the log
         self.assert_log_generator(self.auth_client, log_name, lines=100000)
         log_lines = len(self._get_last_log_contents(log_name).splitlines())
@@ -582,9 +626,15 @@ class GuestLogRunner(TestRunner):
         # generate the file
         self.report.log("Executing log_save for log '%s' (publish: %s)" %
                         (log_name, publish))
+
+        if publish:
+            client.instances.log_action(self.instance_info.id,
+                                        log_name=log_name,
+                                        publish=True)
+
         with tempfile.NamedTemporaryFile() as temp_file:
             client.instances.log_save(self.instance_info.id,
-                                      log_name=log_name, publish=publish,
+                                      log_name=log_name,
                                       filename=temp_file.name)
             file_contents = operating_system.read_file(temp_file.name)
             # now grab the contents ourselves
@@ -613,8 +663,9 @@ class GuestLogRunner(TestRunner):
                 expected_published=0, expected_pending=1)
 
     def run_test_log_show_after_stop_details(self):
+        log_name = self._get_exposed_user_log_name()
         self.stopped_log_details = self.auth_client.instances.log_show(
-            self.instance_info.id, self._get_exposed_user_log_name())
+            self.instance_info.id, log_name)
         self.assert_is_not_none(self.stopped_log_details)
 
     def run_test_add_data_again_after_stop(self):
@@ -625,8 +676,9 @@ class GuestLogRunner(TestRunner):
         self.test_helper.verify_data(DataType.micro3, self.get_instance_host())
 
     def run_test_log_show_after_stop(self):
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_show(
-            self.auth_client, self._get_exposed_user_log_name(),
+            self.auth_client, log_name,
             expected_published=self.stopped_log_details.published,
             expected_pending=self.stopped_log_details.pending)
 
@@ -636,9 +688,10 @@ class GuestLogRunner(TestRunner):
         if self.test_helper.log_enable_requires_restart():
             expected_status = guest_log.LogStatus.Restart_Required.name
 
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_enable(
             self.auth_client,
-            self._get_exposed_user_log_name(),
+            log_name,
             expected_status=expected_status,
             expected_published=0, expected_pending=expected_pending)
 
@@ -654,27 +707,33 @@ class GuestLogRunner(TestRunner):
         self.assert_log_publish(
             self.auth_client,
             log_name,
-            expected_status=guest_log.LogStatus.Published.name,
+            expected_status=[guest_log.LogStatus.Published.name,
+                             guest_log.LogStatus.Partial.name],
             expected_published=self._get_last_log_published(log_name) + 1,
-            expected_pending=0)
+            expected_pending=None)
 
     def run_test_log_disable_user_after_stop_start(self):
         expected_status = guest_log.LogStatus.Disabled.name
         if self.test_helper.log_enable_requires_restart():
             expected_status = guest_log.LogStatus.Restart_Required.name
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_disable(
             self.auth_client,
-            self._get_exposed_user_log_name(), discard=True,
+            log_name, discard=True,
             expected_status=expected_status,
             expected_published=0, expected_pending=1)
 
     def run_test_log_show_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_show(
             self.admin_client,
-            self._get_unexposed_sys_log_name(),
+            log_name,
             expected_type=guest_log.LogType.SYS.name,
-            expected_status=guest_log.LogStatus.Ready.name,
-            expected_published=0, expected_pending=1)
+            expected_status=[guest_log.LogStatus.Ready.name,
+                             guest_log.LogStatus.Partial.name],
+            expected_published=0, expected_pending=1,
+            is_admin=True
+        )
 
     def run_test_log_publish_sys(self):
         log_name = self._get_unexposed_sys_log_name()
@@ -682,8 +741,10 @@ class GuestLogRunner(TestRunner):
             self.admin_client,
             log_name,
             expected_type=guest_log.LogType.SYS.name,
-            expected_status=guest_log.LogStatus.Partial.name,
-            expected_published=1, expected_pending=1)
+            expected_status=[guest_log.LogStatus.Partial.name,
+                             guest_log.LogStatus.Published.name],
+            expected_published=1, expected_pending=None,
+            is_admin=True)
 
     def run_test_log_publish_again_sys(self):
         log_name = self._get_unexposed_sys_log_name()
@@ -691,44 +752,52 @@ class GuestLogRunner(TestRunner):
             self.admin_client,
             log_name,
             expected_type=guest_log.LogType.SYS.name,
-            expected_status=guest_log.LogStatus.Partial.name,
+            expected_status=[guest_log.LogStatus.Partial.name,
+                             guest_log.LogStatus.Published.name],
             expected_published=self._get_last_log_published(log_name) + 1,
-            expected_pending=1)
+            expected_pending=None,
+            is_admin=True)
 
     def run_test_log_generator_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_generator(
             self.admin_client,
-            self._get_unexposed_sys_log_name(),
+            log_name,
             lines=4, expected_lines=4)
 
     def run_test_log_generator_publish_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_generator(
             self.admin_client,
-            self._get_unexposed_sys_log_name(), publish=True,
+            log_name, publish=True,
             lines=4, expected_lines=4)
 
     def run_test_log_generator_swift_client_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_generator(
             self.admin_client,
-            self._get_unexposed_sys_log_name(), publish=True,
+            log_name, publish=True,
             lines=4, expected_lines=4,
-            swift_client=self.swift_client)
+            swift_client=self.admin_swift_client)
 
     def run_test_log_save_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_test_log_save(
             self.admin_client,
-            self._get_unexposed_sys_log_name())
+            log_name)
 
     def run_test_log_save_publish_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_test_log_save(
             self.admin_client,
-            self._get_unexposed_sys_log_name(),
+            log_name,
             publish=True)
 
     def run_test_log_discard_sys(self):
+        log_name = self._get_unexposed_sys_log_name()
         self.assert_log_discard(
             self.admin_client,
-            self._get_unexposed_sys_log_name(),
+            log_name,
             expected_type=guest_log.LogType.SYS.name,
             expected_status=guest_log.LogStatus.Ready.name,
             expected_published=0, expected_pending=1)
@@ -737,7 +806,8 @@ class GuestLogRunner(TestRunner):
 class CassandraGuestLogRunner(GuestLogRunner):
 
     def run_test_log_show(self):
+        log_name = self._get_exposed_user_log_name()
         self.assert_log_show(self.auth_client,
-                             self._get_exposed_user_log_name(),
+                             log_name,
                              expected_published=0,
                              expected_pending=None)
