@@ -14,12 +14,15 @@
 #    under the License.
 #
 
-from mock import Mock
+from unittest.mock import Mock
+from unittest.mock import patch
 
 from testtools import ExpectedException
 from trove.common import exception
 from trove.common import utils
 from trove.tests.unittests import trove_testtools
+from trove.tests.util import utils as test_utils
+import webob
 
 
 class TestUtils(trove_testtools.TestCase):
@@ -61,8 +64,10 @@ class TestUtils(trove_testtools.TestCase):
                 '/usr/bin/foo', log_output_on_error=True)
 
         utils.LOG.error.assert_called_with(
-            u"Command 'test' failed. test-desc Exit code: 42\n"
-            "stderr: err\nstdout: out")
+            u"Command '%(cmd)s' failed. %(description)s Exit code: "
+            u"%(exit_code)s\nstderr: %(stderr)s\nstdout: %(stdout)s",
+            {'description': 'test-desc', 'stderr': 'err', 'exit_code': 42,
+             'stdout': 'out', 'cmd': 'test'})
 
     def test_unpack_singleton(self):
         self.assertEqual([1, 2, 3], utils.unpack_singleton([1, 2, 3]))
@@ -123,3 +128,77 @@ class TestUtils(trove_testtools.TestCase):
     def test_to_mb_zero(self):
         result = utils.to_mb(0)
         self.assertEqual(0.0, result)
+
+    @patch('trove.common.utils.LOG')
+    def test_retry_decorator(self, _):
+
+        class TestEx1(Exception):
+            pass
+
+        class TestEx2(Exception):
+            pass
+
+        class TestEx3(Exception):
+            pass
+
+        class TestExecutor(object):
+
+            def _test_foo(self, arg):
+                return arg
+
+            @test_utils.retry(TestEx1, retries=5, delay_fun=lambda n: 0.2)
+            def test_foo_1(self, arg):
+                return self._test_foo(arg)
+
+            @test_utils.retry((TestEx1, TestEx2), delay_fun=lambda n: 0.2)
+            def test_foo_2(self, arg):
+                return self._test_foo(arg)
+
+        def assert_retry(fun, side_effect, exp_call_num, exp_exception):
+            with patch.object(te, '_test_foo', side_effect=side_effect) as f:
+                mock_arg = Mock()
+                if exp_exception:
+                    self.assertRaises(exp_exception, fun, mock_arg)
+                else:
+                    fun(mock_arg)
+
+                f.assert_called_with(mock_arg)
+                self.assertEqual(exp_call_num, f.call_count)
+
+        te = TestExecutor()
+        assert_retry(te.test_foo_1, [TestEx1, None], 2, None)
+        assert_retry(te.test_foo_1, TestEx3, 1, TestEx3)
+        assert_retry(te.test_foo_1, TestEx1, 5, TestEx1)
+        assert_retry(te.test_foo_1, [TestEx1, TestEx3], 2, TestEx3)
+        assert_retry(te.test_foo_2, [TestEx1, TestEx2, None], 3, None)
+        assert_retry(te.test_foo_2, TestEx3, 1, TestEx3)
+        assert_retry(te.test_foo_2, TestEx2, 3, TestEx2)
+        assert_retry(te.test_foo_2, [TestEx1, TestEx3, TestEx2], 2, TestEx3)
+
+    def test_req_to_text(self):
+        req = webob.Request.blank('/')
+        expected = 'GET / HTTP/1.0\r\nHost: localhost:80'
+        self.assertEqual(expected, utils.req_to_text(req))
+
+        # add a header containing unicode characters
+        req.headers.update({
+            'X-Auth-Project-Id': '\u6d4b\u8bd5'})
+        expected = ('GET / HTTP/1.0\r\nHost: localhost:80\r\n'
+                    'X-Auth-Project-Id: \u6d4b\u8bd5')
+        self.assertEqual(expected, utils.req_to_text(req))
+
+    def test_validate_command(self):
+        string1 = "hello_world"
+        string2 = "hello world"
+        string3 = "hello@world_123"
+        string4 = "example.com/databse/mysql:5.7"
+        string5 = 'test --db-user="$(touch /rce_successful.txt)"'
+        self.assertIsNone(utils.validate_command(string1))
+        self.assertRaises(exception.InvalidValue,
+                          utils.validate_command,
+                          string2)
+        self.assertIsNone(utils.validate_command(string3))
+        self.assertIsNone(utils.validate_command(string4))
+        self.assertRaises(exception.InvalidValue,
+                          utils.validate_command,
+                          string5)
