@@ -13,20 +13,23 @@
 #    under the License.
 import uuid
 
-from mock import Mock, patch
+from unittest.mock import Mock
+from unittest.mock import patch
 
 from trove.backup import models as backup_models
 from trove.common import cfg
+from trove.common import clients
 from trove.common import exception
-from trove.common.instance import ServiceStatuses
+from trove.common import neutron
 from trove.datastore import models as datastore_models
 from trove.instance import models
 from trove.instance.models import DBInstance
 from trove.instance.models import DBInstanceFault
-from trove.instance.models import filter_ips
 from trove.instance.models import Instance
 from trove.instance.models import InstanceServiceStatus
 from trove.instance.models import SimpleInstance
+from trove.instance.models import instance_encryption_key_cache
+from trove.instance.service_status import ServiceStatuses
 from trove.instance.tasks import InstanceTasks
 from trove.taskmanager import api as task_api
 from trove.tests.fakes import nova
@@ -48,17 +51,28 @@ class SimpleInstanceTest(trove_testtools.TestCase):
                 ServiceStatuses.BUILDING), ds_version=Mock(), ds=Mock(),
             locality='affinity')
         self.instance.context = self.context
-        db_info.addresses = {"private": [{"addr": "123.123.123.123"}],
-                             "internal": [{"addr": "10.123.123.123"}],
-                             "public": [{"addr": "15.123.123.123"}]}
-        self.orig_conf = CONF.network_label_regex
+        db_info.addresses = [{
+            'type': 'private',
+            'address': '123.123.123.123',
+            'network': 'net-id-private'}, {
+            'type': 'private',
+            'address': '10.123.123.123',
+            'network': 'net-id-private'}, {
+            'type': 'public',
+            'address': '15.123.123.123',
+            'network': 'net-id-public'}]
+
         self.orig_ip_regex = CONF.ip_regex
         self.orig_black_list_regex = CONF.black_list_regex
 
     def tearDown(self):
         super(SimpleInstanceTest, self).tearDown()
-        CONF.network_label_regex = self.orig_conf
         CONF.ip_start = None
+        CONF.management_networks = []
+        CONF.ip_regex = self.orig_ip_regex
+        CONF.black_list_regex = self.orig_black_list_regex
+
+        neutron.reset_management_networks()
 
     def test_get_root_on_create(self):
         root_on_create_val = Instance.get_root_on_create(
@@ -66,45 +80,19 @@ class SimpleInstanceTest(trove_testtools.TestCase):
         self.assertFalse(root_on_create_val)
 
     def test_filter_ips_white_list(self):
-        CONF.network_label_regex = '.*'
         CONF.ip_regex = '^(15.|123.)'
         CONF.black_list_regex = '^10.123.123.*'
         ip = self.instance.get_visible_ip_addresses()
-        ip = filter_ips(
-            ip, CONF.ip_regex, CONF.black_list_regex)
         self.assertEqual(2, len(ip))
-        self.assertTrue('123.123.123.123' in ip)
-        self.assertTrue('15.123.123.123' in ip)
+        self.assertIn('123.123.123.123', ip[0].get('address'))
+        self.assertIn('15.123.123.123', ip[1].get('address'))
 
     def test_filter_ips_black_list(self):
-        CONF.network_label_regex = '.*'
         CONF.ip_regex = '.*'
         CONF.black_list_regex = '^10.123.123.*'
         ip = self.instance.get_visible_ip_addresses()
-        ip = filter_ips(
-            ip, CONF.ip_regex, CONF.black_list_regex)
         self.assertEqual(2, len(ip))
-        self.assertTrue('10.123.123.123' not in ip)
-
-    def test_one_network_label(self):
-        CONF.network_label_regex = 'public'
-        ip = self.instance.get_visible_ip_addresses()
-        self.assertEqual(['15.123.123.123'], ip)
-
-    def test_two_network_labels(self):
-        CONF.network_label_regex = '^(private|public)$'
-        ip = self.instance.get_visible_ip_addresses()
-        self.assertEqual(2, len(ip))
-        self.assertTrue('123.123.123.123' in ip)
-        self.assertTrue('15.123.123.123' in ip)
-
-    def test_all_network_labels(self):
-        CONF.network_label_regex = '.*'
-        ip = self.instance.get_visible_ip_addresses()
-        self.assertEqual(3, len(ip))
-        self.assertTrue('10.123.123.123' in ip)
-        self.assertTrue('123.123.123.123' in ip)
-        self.assertTrue('15.123.123.123' in ip)
+        self.assertNotIn('10.123.123.123', ip)
 
     def test_locality(self):
         self.assertEqual('affinity', self.instance.locality)
@@ -182,11 +170,11 @@ class CreateInstanceTest(trove_testtools.TestCase):
             datastore_version_id=self.datastore_version.id,
             deleted=False
         )
-        self.backup.size = 1.1
-        self.backup.save()
         self.backup_id = self.backup.id
-        self.orig_client = models.create_nova_client
-        models.create_nova_client = nova.fake_create_nova_client
+
+        self.orig_client = clients.create_nova_client
+        clients.create_nova_client = nova.fake_create_nova_client
+
         self.orig_api = task_api.API(self.context).create_instance
         task_api.API(self.context).create_instance = Mock()
         self.run_with_quotas = models.run_with_quotas
@@ -195,6 +183,12 @@ class CreateInstanceTest(trove_testtools.TestCase):
         backup_models.DBBackup.check_swift_object_exist = Mock(
             return_value=True)
         self.locality = 'affinity'
+
+        self.swift_verify_patch = patch.object(models.Backup,
+                                               'verify_swift_auth_token')
+        self.addCleanup(self.swift_verify_patch.stop)
+        self.swift_verify_patch.start()
+
         super(CreateInstanceTest, self).setUp()
 
     @patch.object(task_api.API, 'get_client', Mock(return_value=Mock()))
@@ -203,7 +197,7 @@ class CreateInstanceTest(trove_testtools.TestCase):
         self.backup.delete()
         self.datastore.delete()
         self.datastore_version.delete()
-        models.create_nova_client = self.orig_client
+        clients.create_nova_client = self.orig_client
         task_api.API(self.context).create_instance = self.orig_api
         models.run_with_quotas = self.run_with_quotas
         backup_models.DBBackup.check_swift_object_exist = self.check
@@ -212,6 +206,8 @@ class CreateInstanceTest(trove_testtools.TestCase):
         super(CreateInstanceTest, self).tearDown()
 
     def test_exception_on_invalid_backup_size(self):
+        self.backup.size = 1.1
+        self.backup.save()
         self.assertEqual(self.backup.id, self.backup_id)
         exc = self.assertRaises(
             exception.BackupTooLarge, models.Instance.create,
@@ -279,20 +275,22 @@ class TestInstanceUpgrade(trove_testtools.TestCase):
             manager='test',
             active=1)
 
-        self.safe_nova_client = models.create_nova_client
-        models.create_nova_client = nova.fake_create_nova_client
+        self.safe_nova_client = clients.create_nova_client
+        clients.create_nova_client = nova.fake_create_nova_client
         super(TestInstanceUpgrade, self).setUp()
 
     def tearDown(self):
         self.datastore.delete()
         self.datastore_version1.delete()
         self.datastore_version2.delete()
-        models.create_nova_client = self.safe_nova_client
+        clients.create_nova_client = self.safe_nova_client
         super(TestInstanceUpgrade, self).tearDown()
 
+    @patch('trove.common.clients.create_neutron_client')
     @patch.object(task_api.API, 'get_client', Mock(return_value=Mock()))
     @patch.object(task_api.API, 'upgrade')
-    def test_upgrade(self, task_upgrade):
+    @patch('trove.tests.fakes.nova.LOG')
+    def test_upgrade(self, mock_logging, task_upgrade, mock_neutron_client):
         instance_model = DBInstance(
             InstanceTasks.NONE,
             id=str(uuid.uuid4()),
@@ -341,11 +339,17 @@ class TestReplication(trove_testtools.TestCase):
             manager='mysql',
             active=1)
 
+        self.databases = []
+
+        self.users = []
+
         self.master = DBInstance(
             InstanceTasks.NONE,
             id=str(uuid.uuid4()),
             name="TestMasterInstance",
-            datastore_version_id=self.datastore_version.id)
+            datastore_version_id=self.datastore_version.id,
+            flavor_id=str(uuid.uuid4()),
+            volume_size=2)
         self.master.set_task_status(InstanceTasks.NONE)
         self.master.save()
         self.master_status = InstanceServiceStatus(
@@ -354,8 +358,14 @@ class TestReplication(trove_testtools.TestCase):
             instance_id=self.master.id)
         self.master_status.save()
 
-        self.safe_nova_client = models.create_nova_client
-        models.create_nova_client = nova.fake_create_nova_client
+        self.safe_nova_client = clients.create_nova_client
+        clients.create_nova_client = nova.fake_create_nova_client
+
+        self.swift_verify_patch = patch.object(models.Backup,
+                                               'verify_swift_auth_token')
+        self.addCleanup(self.swift_verify_patch.stop)
+        self.swift_verify_patch.start()
+
         super(TestReplication, self).setUp()
 
     def tearDown(self):
@@ -363,42 +373,63 @@ class TestReplication(trove_testtools.TestCase):
         self.master_status.delete()
         self.datastore.delete()
         self.datastore_version.delete()
-        models.create_nova_client = self.safe_nova_client
+        clients.create_nova_client = self.safe_nova_client
         super(TestReplication, self).tearDown()
 
-    @patch('trove.instance.models.LOG')
-    def test_replica_of_not_active_master(self, mock_logging):
-        self.master.set_task_status(InstanceTasks.BUILDING)
-        self.master.save()
-        self.master_status.set_status(ServiceStatuses.BUILDING)
-        self.master_status.save()
-        self.assertRaises(exception.UnprocessableEntity,
-                          Instance.create,
-                          None, 'name', 1, "UUID", [], [], None,
-                          self.datastore_version, 1,
-                          None, slave_of_id=self.master.id)
-
     @patch('trove.instance.models.LOG')
     def test_replica_with_invalid_slave_of_id(self, mock_logging):
         self.assertRaises(exception.NotFound,
                           Instance.create,
-                          None, 'name', 1, "UUID", [], [], None,
-                          self.datastore_version, 1,
+                          None, 'name', 1, "UUID", [], [], self.datastore,
+                          self.datastore_version, 2,
                           None, slave_of_id=str(uuid.uuid4()))
 
-    def test_create_replica_from_replica(self):
-        self.replica_datastore_version = Mock(
-            spec=datastore_models.DBDatastoreVersion)
-        self.replica_datastore_version.id = "UUID"
-        self.replica_datastore_version.manager = 'mysql'
-        self.replica_info = DBInstance(
-            InstanceTasks.NONE,
-            id="UUID",
-            name="TestInstance",
-            datastore_version_id=self.replica_datastore_version.id,
-            slave_of_id=self.master.id)
-        self.replica_info.save()
-        self.assertRaises(exception.Forbidden, Instance.create,
-                          None, 'name', 2, "UUID", [], [], None,
-                          self.datastore_version, 1,
-                          None, slave_of_id=self.replica_info.id)
+
+def trivial_key_function(id):
+    return id * id
+
+
+class TestInstanceKeyCaching(trove_testtools.TestCase):
+
+    def setUp(self):
+        super(TestInstanceKeyCaching, self).setUp()
+
+    def tearDown(self):
+        super(TestInstanceKeyCaching, self).tearDown()
+
+    def test_basic_caching(self):
+        keycache = instance_encryption_key_cache(trivial_key_function, 5)
+        self.assertEqual(keycache[5], 25)
+        self.assertEqual(keycache[5], 25)
+        self.assertEqual(keycache[25], 625)
+
+    def test_caching(self):
+        keyfn = Mock(return_value=123)
+        keycache = instance_encryption_key_cache(keyfn, 5)
+        self.assertEqual(keycache[5], 123)
+        self.assertEqual(keyfn.call_count, 1)
+        self.assertEqual(keycache[5], 123)
+        self.assertEqual(keyfn.call_count, 1)
+        self.assertEqual(keycache[6], 123)
+        self.assertEqual(keyfn.call_count, 2)
+        self.assertEqual(keycache[7], 123)
+        self.assertEqual(keyfn.call_count, 3)
+        self.assertEqual(keycache[8], 123)
+        self.assertEqual(keyfn.call_count, 4)
+        self.assertEqual(keycache[9], 123)
+        self.assertEqual(keyfn.call_count, 5)
+        self.assertEqual(keycache[10], 123)
+        self.assertEqual(keyfn.call_count, 6)
+        self.assertEqual(keycache[10], 123)
+        self.assertEqual(keyfn.call_count, 6)
+        self.assertEqual(keycache[5], 123)
+        self.assertEqual(keyfn.call_count, 7)
+
+    # BUG(1650518): Cleanup in the Pike release
+    def test_not_caching_none(self):
+        keyfn = Mock(return_value=None)
+        keycache = instance_encryption_key_cache(keyfn, 5)
+        self.assertIsNone(keycache[30])
+        self.assertEqual(keyfn.call_count, 1)
+        self.assertIsNone(keycache[30])
+        self.assertEqual(keyfn.call_count, 2)
